@@ -16,7 +16,6 @@
  */
 
 #include <curses.h>
-#include <pthread.h>
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
@@ -27,27 +26,6 @@
 #define OUTPUT_HEIGHT LINES - 7
 #define INPUT_HEIGHT 6
 
-/*
-  A simple mechanism for enqueuing requests to update the console. Ncurses is
-  not (easily) thread-safe, so this ensures that all calls to the Ncurses API
-  and which may mutate Ncurses global state originate from the same thread.
- */
-
-typedef enum 
-  {
-    ABERMUD_CONSOLE_NONE,
-    ABERMUD_CONSOLE_APPEND_OUTPUT,
-    ABERMUD_CONSOLE_SET_TITLE,
-    ABERMUD_CONSOLE_SET_PROMPT
-  }
-  abermud_console_event_type;
-
-static pthread_mutex_t console_event_lock = PTHREAD_MUTEX_INITIALIZER;
-static pthread_cond_t console_event_cond = PTHREAD_COND_INITIALIZER;
-
-static abermud_console_event_type console_event_type = ABERMUD_CONSOLE_NONE;
-static char *console_event_text = NULL;
-
 /* 
    Initialize the console and return a pointer to a new console state object
    that shares input state with the game context via the abermud_input_buffer 
@@ -73,12 +51,8 @@ abermud_console_init (abermud_input_buffer *input_buffer)
   idlok (state->output, TRUE);
   idlok (state->input, TRUE);
   keypad (state->input, TRUE);
-  noecho ();
-
-  /* Limit the amount of time Ncurses will wait for input to 1 tenth of a 
-     second. */
-
-  halfdelay (1);
+  noecho (); 
+  nodelay (state->input, TRUE);
 
   return state;
 }
@@ -89,24 +63,24 @@ abermud_console_init (abermud_input_buffer *input_buffer)
   OUTPUT. 
  */
 
-static void 
-append_output (abermud_console_state *state)
+void 
+abermud_console_append_output (abermud_console_state *state, char *output)
 {
-  int i = 0, len = strlen (console_event_text);
+  int i = 0, len = strlen (output);
   
   scroll (state->output);
   wmove (state->output, OUTPUT_HEIGHT - 1, 0);
   
   for (; i < len; i++) 
     {
-      if (console_event_text [i] == '\n')
+      if (output [i] == '\n')
 	{
 	  scroll (state->output);
 	  wmove (state->output, OUTPUT_HEIGHT - 1, 0);
 	  wclrtoeol (state->output);
 	  wmove (state->output, OUTPUT_HEIGHT - 1, 0);
 	}
-      else waddch (state->output, console_event_text[i]);
+      else waddch (state->output, output[i]);
     }
 
   wrefresh (state->output);
@@ -117,24 +91,6 @@ append_output (abermud_console_state *state)
   abermud_console_input_cursor_to (state, state->input_buffer->pos);
 }
 
-/*
-  Enqueue a request to add output to the console.
- */
-
-void 
-abermud_console_append_output (abermud_console_state *state, char *output)
-{
-  pthread_mutex_lock (&console_event_lock);
-  
-  while (console_event_type != ABERMUD_CONSOLE_NONE)
-    pthread_cond_wait (&console_event_cond, &console_event_lock);
-
-  console_event_type = ABERMUD_CONSOLE_APPEND_OUTPUT;
-  console_event_text = strdup (output);
-
-  pthread_mutex_unlock (&console_event_lock);
-}
-
 /*
   Sets the console's input prompt to the contents of PROMPT and redraws the 
   input window, including the current contents of the input buffer. The 
@@ -142,26 +98,9 @@ abermud_console_append_output (abermud_console_state *state, char *output)
   necessary. 
  */
 
-static void 
-set_prompt (abermud_console_state *state)
-{
-  werase (state->input);
-  wprintw (state->input, "%s", state->prompt);
-  wmove (state->input, 0, strlen (state->prompt));
-  wprintw (state->input, "%s", state->input_buffer->data);
-  abermud_console_input_cursor_to (state, state->input_buffer->pos);
-  wrefresh (state->input);
-}
-
-/*
-  Enqueue a request to set the console prompt.
- */
-
 void 
 abermud_console_set_prompt (abermud_console_state *state, char *prompt)
 {
-  pthread_mutex_lock (&console_event_lock);
-  
   /* Free any existing prompt data. */
 
   if (state->prompt != NULL)
@@ -169,11 +108,12 @@ abermud_console_set_prompt (abermud_console_state *state, char *prompt)
 
   state->prompt = strndup (prompt, strlen (prompt));
 
-  while (console_event_type != ABERMUD_CONSOLE_NONE)
-    pthread_cond_wait (&console_event_cond, &console_event_lock);
-
-  console_event_type = ABERMUD_CONSOLE_SET_PROMPT;
-  pthread_mutex_unlock (&console_event_lock);
+  werase (state->input);
+  wprintw (state->input, "%s", prompt);
+  wmove (state->input, 0, strlen (prompt));
+  wprintw (state->input, "%s", state->input_buffer->data);
+  abermud_console_input_cursor_to (state, state->input_buffer->pos);
+  wrefresh (state->input);
 }
 
 /*
@@ -181,14 +121,14 @@ abermud_console_set_prompt (abermud_console_state *state, char *prompt)
   contents of TITLE. 
  */
 
-static void 
-set_title (abermud_console_state *state)
+void 
+abermud_console_set_title (abermud_console_state *state, char *title)
 {
-  int i = 0, len = strlen (console_event_text);
+  int i = 0, len = strlen (title);
   
   wattron (state->title, A_REVERSE);
   wclrtoeol (state->title);
-  mvwaddnstr (state->title, 0, 0, console_event_text, len);
+  mvwaddnstr (state->title, 0, 0, title, len);
   
   for (; i < COLS - len; i++) 
     waddch (state->title, ' ');
@@ -197,24 +137,6 @@ set_title (abermud_console_state *state)
   wrefresh (state->title);
 }
 
-/*
-  Enqueue a request to set the title.
- */
-
-void 
-abermud_console_set_title (abermud_console_state *state, char *title)
-{
-  pthread_mutex_lock (&console_event_lock);
-  
-  while (console_event_type != ABERMUD_CONSOLE_NONE)
-    pthread_cond_wait (&console_event_cond, &console_event_lock);
-
-  console_event_type = ABERMUD_CONSOLE_SET_TITLE;
-  console_event_text = strdup (title);
-
-  pthread_mutex_unlock (&console_event_lock);  
-}
-
 /*
   Renders the contents of the console's input area (i.e., everything following
   the prompt) based on the state of the shared input buffer. This function 
@@ -271,45 +193,6 @@ abermud_console_shutdown (abermud_console_state *state)
   free (state);
 }
 
-/*
-  Empty the console event state to indicate that a new event may be enqueued.
- */
-
-static void 
-clear_console_event (void)
-{
-  console_event_type = ABERMUD_CONSOLE_NONE;
-
-  if (console_event_text != NULL)
-    {
-      free (console_event_text);
-      console_event_text = NULL;
-    }
-}
-
-/*
-  Dispatch the enqueued event, if any.
- */
-
-static void 
-dispatch_console_event (abermud_console_state *state)
-{
-  pthread_mutex_lock (&console_event_lock);
-
-  switch (console_event_type)
-    {
-    case ABERMUD_CONSOLE_APPEND_OUTPUT: append_output (state); break;
-    case ABERMUD_CONSOLE_SET_TITLE: set_title (state); break;
-    case ABERMUD_CONSOLE_SET_PROMPT: set_prompt (state); break;
-    default: break;
-    }
-
-  clear_console_event ();
-
-  pthread_cond_signal (&console_event_cond);
-  pthread_mutex_unlock (&console_event_lock);
-}
-
 /*
   Reads and returns a character from the input area (i.e., the input window)
   of the console. Since the Ncurses input system is set to wait for input for a
@@ -320,13 +203,6 @@ dispatch_console_event (abermud_console_state *state)
 int 
 abermud_console_getch (abermud_console_state *state)
 {
-  while (TRUE)
-    {
-      int ret = wgetch (state->input);
-
-      if (ret != ERR)
-	return ret;
-
-      dispatch_console_event (state);
-    }
+  int ret = wgetch (state->input);
+  return ret == ERR ? 0 : ret;
 }
