@@ -1,5 +1,5 @@
 /* Administrative context routines for gzochid
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -40,7 +40,6 @@ struct _gzochid_admin_context
   GzochidRootContext *root_context;
   GzochidHttpServer *http_server;
   
-  GThreadPool *pool;
   GHashTable *config;
 };
 
@@ -57,17 +56,11 @@ initialize (int from_state, int to_state, gpointer user_data)
   gzochid_context *context = user_data;
   gzochid_admin_context *admin_context = (gzochid_admin_context *) context;
 
-  admin_context->pool = 
-    gzochid_thread_pool_new 
-    (context, gzochid_config_to_int 
-     (g_hash_table_lookup 
-      (admin_context->config, "thread_pool.max_threads"), 4), 
-     TRUE, NULL);
-
   if (gzochid_config_to_boolean 
       (g_hash_table_lookup 
        (admin_context->config, "module.httpd.enabled"), FALSE))
     {
+      GError *err = NULL;
       int port = gzochid_config_to_int 
 	(g_hash_table_lookup (admin_context->config, "module.httpd.port"), 
 	 8000);
@@ -77,10 +70,17 @@ initialize (int from_state, int to_state, gpointer user_data)
 	 GZOCHID_TYPE_HTTP_SERVER, NULL);
       
       gzochid_httpd_app_register_handlers
-	(admin_context->http_server, (gzochid_game_context *)
-	 admin_context->root_context->game_server);
+	(admin_context->http_server, admin_context->root_context->game_server,
+	 admin_context->root_context->resolution_context);
 
-      gzochid_http_server_start (admin_context->http_server, port, NULL);
+      gzochid_http_server_start (admin_context->http_server, port, &err);
+
+      if (err != NULL)
+	{
+	  g_critical
+	    ("Failed to start admin HTTP server: %s; exiting...", err->message);
+	  exit (EXIT_FAILURE);
+	}
     }
   if (gzochid_config_to_boolean 
       (g_hash_table_lookup 
