@@ -1,5 +1,5 @@
 /* channel.c: Primitive functions for user-facing gzochid channel API
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -34,15 +34,27 @@ SCM_DEFINE (primitive_create_channel, "primitive-create-channel", 1, 0, 0,
   gzochid_application_context *context = 
     gzochid_api_ensure_current_application_context ();
   char *cname = scm_to_locale_string (name);
-  gzochid_channel *channel = gzochid_channel_create (context, cname);
-  gzochid_data_managed_reference *scm_reference = 
-    gzochid_data_create_reference_to_oid 
-    (context, &gzochid_scheme_data_serialization, channel->scm_oid);
+  gzochid_channel *channel = gzochid_channel_create (context, cname);  
+  gzochid_data_managed_reference *scm_reference = NULL;
+  SCM ret = SCM_BOOL_F;
 
-  gzochid_data_dereference (scm_reference, NULL);
+  /* `gzochid_channel_create' may return `NULL' if the transaction has gotten
+     into a bad state. */
+  
+  if (channel != NULL)
+    {
+      guint64 scm_oid = gzochid_channel_scm_oid (channel);
+  
+      scm_reference = gzochid_data_create_reference_to_oid 
+	(context, &gzochid_scheme_data_serialization, scm_oid);
+  
+      gzochid_data_dereference (scm_reference, NULL);      
+      ret = scm_reference->obj;
+    }
+
+  free (cname);
   gzochid_api_check_transaction ();
-
-  return (SCM) scm_reference->obj;
+  return ret;
 }
 
 SCM_DEFINE (primitive_get_channel, "primitive-get-channel", 1, 0, 0,
@@ -58,9 +70,11 @@ SCM_DEFINE (primitive_get_channel, "primitive-get-channel", 1, 0, 0,
   free (cname);
   if (channel != NULL)
     {
-      gzochid_data_managed_reference *scm_reference = 
+      guint64 scm_oid = gzochid_channel_scm_oid (channel);
+      gzochid_data_managed_reference *scm_reference =
 	gzochid_data_create_reference_to_oid 
-	(context, &gzochid_scheme_data_serialization, channel->scm_oid);
+	(context, &gzochid_scheme_data_serialization, scm_oid);
+
       gzochid_data_dereference (scm_reference, &err);
 
       if (err == NULL)
@@ -76,27 +90,18 @@ SCM_DEFINE (primitive_get_channel, "primitive-get-channel", 1, 0, 0,
 SCM_DEFINE (primitive_join_channel, "primitive-join-channel", 2, 0, 0,
 	    (SCM channel, SCM session), "Add client session to a channel.")
 {
+  GError *err = NULL;
   gzochid_application_context *context = 
     gzochid_api_ensure_current_application_context ();
-  gzochid_data_managed_reference *channel_reference = NULL;
-  gzochid_data_managed_reference *session_reference = NULL;
-  mpz_t channel_oid, session_oid;
-  GError *err = NULL;
-
-  mpz_init (channel_oid);
-  mpz_init (session_oid);
-  
-  gzochid_scheme_channel_oid (channel, channel_oid);
-  gzochid_scheme_client_session_oid (session, session_oid);
-  
-  channel_reference = gzochid_data_create_reference_to_oid 
+  guint64 channel_oid = gzochid_scheme_channel_oid (channel);
+  guint64 session_oid = gzochid_scheme_client_session_oid (session);
+  gzochid_data_managed_reference *channel_reference =
+    gzochid_data_create_reference_to_oid 
     (context, &gzochid_channel_serialization, channel_oid);
-  session_reference = gzochid_data_create_reference_to_oid
+  gzochid_data_managed_reference *session_reference =
+    gzochid_data_create_reference_to_oid
     (context, &gzochid_client_session_serialization, session_oid);
 
-  mpz_clear (channel_oid);
-  mpz_clear (session_oid);
-
   gzochid_data_dereference (channel_reference, &err);
   if (err == NULL)
     {
@@ -121,25 +126,16 @@ SCM_DEFINE (primitive_leave_channel, "primitive-leave-channel", 2, 0, 0,
   GError *err = NULL;
   gzochid_application_context *context = 
     gzochid_api_ensure_current_application_context ();
-  gzochid_data_managed_reference *channel_reference = NULL;
-  gzochid_data_managed_reference *session_reference = NULL;
-  mpz_t channel_oid, session_oid;
-
-  mpz_init (channel_oid);
-  mpz_init (session_oid);
-  
-  gzochid_scheme_channel_oid (channel, channel_oid);
-  gzochid_scheme_client_session_oid (session, session_oid);
-  
-  channel_reference = gzochid_data_create_reference_to_oid 
+  guint64 channel_oid = gzochid_scheme_channel_oid (channel);
+  guint64 session_oid = gzochid_scheme_client_session_oid (session);  
+  gzochid_data_managed_reference *channel_reference =
+    gzochid_data_create_reference_to_oid 
     (context, &gzochid_channel_serialization, channel_oid);
-  session_reference = gzochid_data_create_reference_to_oid
+  gzochid_data_managed_reference *session_reference =
+    gzochid_data_create_reference_to_oid
     (context, &gzochid_client_session_serialization, session_oid);
   
   gzochid_data_dereference (channel_reference, &err);
-
-  mpz_clear (session_oid);
-  mpz_clear (channel_oid);
   
   if (err == NULL)
     {
@@ -167,16 +163,10 @@ SCM_DEFINE (primitive_send_channel_message, "primitive-send-channel-message",
   short len = (short) SCM_BYTEVECTOR_LENGTH (bv);
   unsigned char *msg = (unsigned char *) SCM_BYTEVECTOR_CONTENTS (bv);
 
-  gzochid_data_managed_reference *channel_reference = NULL;
-  mpz_t channel_oid;
-
-  mpz_init (channel_oid);
-  
-  gzochid_scheme_channel_oid (channel, channel_oid);
-  
-  channel_reference = gzochid_data_create_reference_to_oid 
+  guint64 channel_oid = gzochid_scheme_channel_oid (channel);
+  gzochid_data_managed_reference *channel_reference =
+    gzochid_data_create_reference_to_oid 
     (context, &gzochid_channel_serialization, channel_oid);
-  mpz_clear (channel_oid);
 
   gzochid_data_dereference (channel_reference, &err);
 
@@ -195,12 +185,9 @@ SCM_DEFINE (primitive_close_channel, "primitive-close-channel", 1, 0, 0,
   GError *err = NULL;
   gzochid_application_context *context = 
     gzochid_api_ensure_current_application_context ();
-  gzochid_data_managed_reference *channel_reference = NULL;
-  mpz_t channel_oid;
-
-  mpz_init (channel_oid);
-  gzochid_scheme_channel_oid (channel, channel_oid);  
-  channel_reference = gzochid_data_create_reference_to_oid 
+  guint64 channel_oid = gzochid_scheme_channel_oid (channel);  
+  gzochid_data_managed_reference *channel_reference =
+    gzochid_data_create_reference_to_oid 
     (context, &gzochid_channel_serialization, channel_oid);
 
   gzochid_data_dereference (channel_reference, &err);
