@@ -38,8 +38,7 @@ SCM_DEFINE (primitive_schedule_task, "primitive-schedule-task", 1, 2, 0,
 {
   gzochid_application_context *context = 
     gzochid_api_ensure_current_application_context (); 
-  gzochid_auth_identity *identity =
-    gzochid_auth_identity_ref (gzochid_get_current_identity());
+  gzochid_auth_identity *identity = gzochid_get_current_identity();
   gzochid_application_task *scheme_task = gzochid_application_task_new 
     (context, identity, gzochid_scheme_application_task_worker, callback);
 
@@ -51,8 +50,10 @@ SCM_DEFINE (primitive_schedule_task, "primitive-schedule-task", 1, 2, 0,
   if (scm_is_false (delay) || scm_is_eq (delay, SCM_UNDEFINED))
     {
       gzochid_schedule_durable_task 
-	(context, identity, scheme_task, &gzochid_scheme_task_serialization);
-      
+	(context, identity, scheme_task, &gzochid_scheme_task_serialization,
+	 NULL);
+
+      gzochid_application_task_unref (scheme_task);      
       gzochid_api_check_transaction ();
       
       return SCM_UNSPECIFIED;
@@ -69,8 +70,9 @@ SCM_DEFINE (primitive_schedule_task, "primitive-schedule-task", 1, 2, 0,
 	{
 	  gzochid_schedule_delayed_durable_task 
 	    (context, identity, scheme_task, 
-	     &gzochid_scheme_task_serialization, delay_tv);
-	  
+	     &gzochid_scheme_task_serialization, delay_tv, NULL);
+
+	  gzochid_application_task_unref (scheme_task);      
 	  gzochid_api_check_transaction ();
 	  
 	  return SCM_UNSPECIFIED;
@@ -88,8 +90,10 @@ SCM_DEFINE (primitive_schedule_task, "primitive-schedule-task", 1, 2, 0,
 	  
 	  handle = gzochid_schedule_periodic_durable_task 
 	    (context, identity, scheme_task, 
-	     &gzochid_scheme_task_serialization, delay_tv, period_tv);
+	     &gzochid_scheme_task_serialization, delay_tv, period_tv, NULL);
 
+	  gzochid_application_task_unref (scheme_task);
+	  
 	  /* Scheduling a durable task can fail, since it depends on the health
 	     of the current transaction... */	     
 	  
