@@ -1,5 +1,5 @@
 /* app-task.c: Support for application-bound transactional task execution
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@
 struct _gzochid_event_transaction_context
 {
   gzochid_application_context *app_context;
-  struct timeval start_time;
+  guint64 start_time;
 };
 
 typedef struct _gzochid_event_transaction_context
@@ -178,7 +178,7 @@ create_transaction_context (gzochid_application_context *app_context)
     malloc (sizeof (gzochid_event_transaction_context));
 
   tx_context->app_context = app_context;
-  gettimeofday (&tx_context->start_time, NULL);
+  tx_context->start_time = g_get_monotonic_time ();
 
   return tx_context;
 }
@@ -193,14 +193,13 @@ static void
 event_commit (gpointer data)
 {
   gzochid_event_transaction_context *tx_context = data;
-  struct timeval now, duration;
-
-  gettimeofday (&now, NULL);
-  timersub (&now, &tx_context->start_time, &duration);
-
-  gzochid_application_event_dispatch 
+  guint64 duration_us = g_get_monotonic_time () - tx_context->start_time;
+  
+  gzochid_event_dispatch
     (tx_context->app_context->event_source,
-     gzochid_application_transaction_event_new (TRANSACTION_COMMIT, duration));
+     g_object_new (GZOCHID_TYPE_TRANSACTION_EVENT,
+		   "type", TRANSACTION_COMMIT, "duration-us", duration_us,
+		   NULL));
 
   free (tx_context);
 }
@@ -209,15 +208,13 @@ static void
 event_rollback (gpointer data)
 {
   gzochid_event_transaction_context *tx_context = data;
-  struct timeval now, duration;
-
-  gettimeofday (&now, NULL);
-  timersub (&now, &tx_context->start_time, &duration);
+  guint64 duration_us = g_get_monotonic_time () - tx_context->start_time;
   
-  gzochid_application_event_dispatch 
+  gzochid_event_dispatch
     (tx_context->app_context->event_source,
-     gzochid_application_transaction_event_new
-     (TRANSACTION_ROLLBACK, duration));
+     g_object_new (GZOCHID_TYPE_TRANSACTION_EVENT,
+		   "type", TRANSACTION_ROLLBACK, "duration-us", duration_us,
+		   NULL));
 
   free (tx_context);
 }
@@ -244,12 +241,13 @@ event_func_wrapper (gpointer data)
   gzochid_application_context *context = args[0];
   void (*func) (gpointer) = (void (*) (gpointer)) args[1];
   gpointer func_data = args[2];
-
+  
   join_transaction (context);
 
-  gzochid_application_event_dispatch
-    (context->event_source, gzochid_application_event_new (TRANSACTION_START));
-
+  gzochid_event_dispatch
+    (context->event_source,
+     g_object_new (GZOCHID_TYPE_EVENT, "type", TRANSACTION_START, NULL));
+     
   func (func_data);  
 }
 
@@ -324,10 +322,6 @@ gzochid_application_resubmitting_transactional_task_worker
 {
   gboolean resubmitting_execution = FALSE;
   gzochid_application_task *application_task = NULL;
-
-  gzochid_context *context = (gzochid_context *) app_context;
-  gzochid_game_context *game_context = (gzochid_game_context *) context->parent;
-
   gzochid_transactional_application_task_execution *execution = data;
 
   gzochid_application_transactional_task_worker 
@@ -374,7 +368,7 @@ gzochid_application_resubmitting_transactional_task_worker
       task.data = application_task;
       gettimeofday (&task.target_execution_time, NULL);
       
-      gzochid_schedule_submit_task (game_context->task_queue, &task);
+      gzochid_schedule_submit_task (app_context->task_queue, &task);
     }
 }
 
