@@ -1,5 +1,5 @@
 /* app.h: Prototypes and declarations for app.c
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -21,51 +21,23 @@
 #include <glib.h>
 
 #include "auth_int.h"
-#include "callback.h"
-#include "context.h"
+#include "descriptor.h"
 #include "event.h"
 #include "gzochid-auth.h"
 #include "gzochid-storage.h"
+#include "metaclient.h"
 #include "oids.h"
+#include "schedule.h"
 #include "stats.h"
 #include "tx.h"
 
-enum gzochid_application_state 
-  {
-    GZOCHID_APPLICATION_STATE_INITIALIZING,
-    GZOCHID_APPLICATION_STATE_RUNNING,
-    GZOCHID_APPLICATION_STATE_PAUSED,
-    GZOCHID_APPLICATION_STATE_STOPPED
-  };
-
-struct _gzochid_application_descriptor
-{
-  char *name;
-  char *description;
-
-  GList *load_paths; /* Descriptor-specified module load paths. */
- 
-  gzochid_application_callback *initialized;
-  gzochid_application_callback *logged_in;
-  gzochid_application_callback *ready;
-
-  char *auth_type;
-  GHashTable *auth_properties;
-
-  GHashTable *properties;
-};
-
-typedef struct _gzochid_application_descriptor gzochid_application_descriptor;
-
 struct _gzochid_application_context
 {
-  gzochid_context base;
-
   /* The directory containing the application descriptor. Used to resolve
      relative load paths. */
 
   char *deployment_root;
-  gzochid_application_descriptor *descriptor;  
+  GzochidApplicationDescriptor *descriptor;
 
   GList *load_paths; /* Complete set of module load paths. */
 
@@ -78,12 +50,17 @@ struct _gzochid_application_context
   GList *free_oid_blocks;
   GMutex free_oids_lock;
 
+  gzochid_storage_engine_interface *storage_engine_interface;
+  
   gzochid_storage_context *storage_context;
   gzochid_storage_store *meta;
   gzochid_storage_store *oids;
   gzochid_storage_store *names;
 
   gzochid_oid_allocation_strategy *oid_strategy;
+
+  gzochid_task_queue *task_queue;
+  struct timeval tx_timeout;
   
   GHashTable *oids_to_clients;
   GHashTable *clients_to_oids;
@@ -95,31 +72,27 @@ struct _gzochid_application_context
 
   GMutex channel_mapping_lock; /* Protects the channel oid mapping. */
   
-  gzochid_application_event_source *event_source;
+  gzochid_event_source *event_source;
   gzochid_application_stats *stats;
+
+  /* Reference to the metaclient, if available. Otherwise, `NULL'. */
+
+  GzochidMetaClient *metaclient; 
 };
 
 typedef struct _gzochid_application_context gzochid_application_context;
 
-/* A helper macro to simplify access to the storage interface of the storage 
-   engine loaded by the game manager that owns the specified application 
-   context.
-
-   Needless to say, this macro assumes that the application context is attached
-   to a properly configured game manager. 
-*/
-
-#define APP_STORAGE_INTERFACE(app_context) \
-  ((gzochid_game_context *) \
-   ((gzochid_context *) (app_context))->parent) \
-  ->storage_engine->interface
-
 gzochid_application_context *gzochid_application_context_new (void);
 
+void gzochid_application_context_init (gzochid_application_context *,
+				       GzochidApplicationDescriptor *,
+				       GzochidMetaClientContainer *,
+				       GzochidAuthPluginRegistry *,
+				       gzochid_storage_engine_interface *,
+				       const char *, gzochid_task_queue *,
+				       struct timeval);
+
 void gzochid_application_context_free (gzochid_application_context *);
-void gzochid_application_context_init (gzochid_application_context *, 
-				       gzochid_context *, 
-				       gzochid_application_descriptor *);
 
 void *gzochid_with_application_context (gzochid_application_context *, 
 					gzochid_auth_identity *,
