@@ -173,10 +173,15 @@ flush_reference (gzochid_data_managed_reference *reference,
 	 (char *) &encoded_oid, sizeof (guint64),
 	 (char *) out->data, out->len);
 
+      /* Need to cast `out->len' to `guint64' to support environments in which 
+	 `guint' is less than 64 bits. */
+      
       gzochid_event_dispatch
 	(context->context->event_source,
 	 g_object_new (GZOCHID_TYPE_DATA_EVENT,
-		       "type", BYTES_WRITTEN, "bytes", out->len, NULL));
+		       "type", BYTES_WRITTEN,
+		       "bytes", (guint64) out->len,
+		       NULL));
       
       g_byte_array_unref (out);
       
@@ -590,11 +595,14 @@ dereference (gzochid_data_transaction_context *context,
   else 
     {
       GError *local_err = NULL;
+
+      /* Need to cast `data_len' to `guint64' since `size_t' may be a different
+	 size on 32-bit platforms. */
       
       gzochid_event_dispatch
 	(context->context->event_source,
 	 g_object_new (GZOCHID_TYPE_DATA_EVENT,
-		       "type", BYTES_READ, "bytes", data_len, NULL));
+		       "type", BYTES_READ, "bytes", (guint64) data_len, NULL));
       
       in = g_byte_array_sized_new (data_len);
       g_byte_array_append (in, (unsigned char *) data, data_len);
