@@ -1,5 +1,5 @@
 /* data.h: Prototypes and declarations for data.c
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 #define GZOCHID_DATA_H
 
 #include <glib.h>
-#include <gmp.h>
 
 #include "app.h"
 #include "gzochid-auth.h"
@@ -36,17 +35,17 @@
 
 GQuark gzochid_data_error_quark (void);
 
-typedef enum
+enum GzochidDataError
   {
     GZOCHID_DATA_ERROR_NOT_FOUND,
     GZOCHID_DATA_ERROR_TRANSACTION,
     GZOCHID_DATA_ERROR_FAILED
-  }
-  GzochidDataError;
+  };
 
 struct _gzochid_oid_holder
 {
-  mpz_t oid;
+  guint64 oid;
+  GError *err;
 };
 
 typedef struct _gzochid_oid_holder gzochid_oid_holder;
@@ -69,21 +68,12 @@ struct _gzochid_data_managed_reference
   enum gzochid_data_managed_reference_state state;
   gzochid_io_serialization *serialization;
 
-  mpz_t oid;
+  guint64 oid;
   void *obj;
 };
 
 typedef struct _gzochid_data_managed_reference gzochid_data_managed_reference;
 
-struct _gzochid_data_oid_block
-{
-  mpz_t first;
-  mpz_t next;
-  mpz_t last;  
-};
-
-typedef struct _gzochid_data_oid_block gzochid_data_oid_block;
-
 struct _gzochid_data_managed_reference_holder
 {
   gzochid_application_context *context;
@@ -106,23 +96,64 @@ void gzochid_data_set_binding
  GError **);
 
 void gzochid_data_set_binding_to_oid
-(gzochid_application_context *, char *, mpz_t, GError **);
+(gzochid_application_context *, char *, guint64, GError **);
 
 void gzochid_data_remove_binding 
 (gzochid_application_context *, char *, GError **);
 
 gboolean gzochid_data_binding_exists 
 (gzochid_application_context *, char *, GError **);
+
 char *gzochid_data_next_binding_oid 
-(gzochid_application_context *, char *, mpz_t, GError **);
+(gzochid_application_context *, char *, guint64 *, GError **);
+
+/*
+  Return a managed reference for the specified pointer within the target gzochi
+  game application context. The data will be serialized via the specified 
+  serialization when the current transaction commits.
+
+  The pointer returned by this function is owned by the data transaction 
+  context, and should not be freed or otherwise modified. This function will 
+  return the same pointer in response to multiple calls for the same pointer
+  within the same transaction.
+
+  The first time this function is called for a particular pointer - i.e., when a
+  new object is being added to the data store - the container attempts to
+  establish a write lock on the new object id. If this causes the transaction to
+  fail (because of a timeout) or if it has already failed, this function will
+  return `NULL' and set the error return argument accordingly.
+*/
 
 gzochid_data_managed_reference *gzochid_data_create_reference
-(gzochid_application_context *, gzochid_io_serialization *, void *);
+(gzochid_application_context *, gzochid_io_serialization *, void *, GError **);
+
 gzochid_data_managed_reference *gzochid_data_create_reference_to_oid
-(gzochid_application_context *, gzochid_io_serialization *, mpz_t);
+(gzochid_application_context *, gzochid_io_serialization *, guint64);
+
+/*
+  Returns the object referenced by the specified managed reference, retrieving 
+  it from the data store and deserializing it if necessary. A read lock will be
+  synchronously established on the object. 
+
+  If the object cannot be retrieved or the lock cannot be established, this 
+  function will return `NULL' and set the error return argument accordingly.
+*/
+
+void *gzochid_data_dereference (gzochid_data_managed_reference *, GError **);
+
+/*
+  Returns the object referenced by the specified managed reference, retrieving 
+  it from the data store and deserializing it if necessary. A write lock will be
+  synchronously established on the object. 
+
+  If the object cannot be retrieved or the write lock cannot be established, 
+  this function will return `NULL' and set the error return argument 
+  accordingly.
+*/
+
+void *gzochid_data_dereference_for_update (gzochid_data_managed_reference *,
+					   GError **);
 
-void gzochid_data_dereference 
-(gzochid_data_managed_reference *reference, GError **);
 void gzochid_data_remove_object (gzochid_data_managed_reference *, GError **);
 void gzochid_data_mark 
 (gzochid_application_context *, gzochid_io_serialization *, void *, GError **);
