@@ -1,5 +1,5 @@
-/* dataclient.c: Data client for gzochid
- * Copyright (C) 2016 Julian Graham
+/* dataclient.c: Dataserver client for gzochid
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -16,25 +16,31 @@
  */
 
 #include <assert.h>
-#include <errno.h>
 #include <glib.h>
 #include <glib-object.h>
 #include <gzochi-common.h>
-#include <netdb.h>
-#include <netinet/in.h>
-#include <netinet/tcp.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/types.h>
-#include <sys/socket.h>
 
 #include "config.h"
 #include "data-protocol.h"
 #include "dataclient.h"
 #include "dataclient-protocol.h"
+#include "log.h"
+#include "meta-protocol.h"
 #include "socket.h"
+#include "util.h"
+
+#ifdef G_LOG_DOMAIN
+#undef G_LOG_DOMAIN
+#endif /* G_LOG_DOMAIN */
+#define G_LOG_DOMAIN "gzochid.dataclient"
+
+#define LOCK_ACCESS(for_write) (for_write ? "r/w" : "read")
+
+#define MAX_MESSAGE_LENGTH (0xffff - 3)
 
 /* Captures callback configuration for a request issued through the data 
    client. */
@@ -57,6 +63,18 @@ struct _dataclient_callback_registration
   gzochid_dataclient_failure_callback failure_callback;
 
   gpointer failure_data; /* Closure data for the failure callback. */
+
+  /* The duration of the point or range lock. Set upon success to the current 
+     monotonic time plus the microsecond-adjusted value of `lock.release.msec'
+     or `rangelock.release.msec', depending. */
+
+  gint64 timeout; 
+
+  /* The release callback. */
+  
+  gzochid_dataclient_release_callback release_callback; 
+
+  gpointer release_data; /* Closure data for the release callback. */
 };
 
 typedef struct _dataclient_callback_registration
@@ -89,43 +107,34 @@ struct _GzochidDataClient
 {
   GObject parent_instance;
 
-  GHashTable *configuration; /* The data client configuration table. */
+  /* The number of milliseconds before the release of a successfully-acquired 
+     point lock will be requested. Set via `lock.release.msec'. */
+
+  unsigned int lock_release_ms;
+
+  /* The number of milliseconds before the release of a successfully-acquired 
+     range lock will be requested. Set via `rangelock.release.msec'. */  
+  
+  unsigned int range_lock_release_ms;
+  
+  GzochidConfiguration *configuration; /* The global configuration object. */
 
   /* A map of application names to `dataclient_callback_queue' objects. */
   
   GHashTable *application_callback_queues; 
 
-  GMutex queue_mutex; /* Synchronizes interactions with the callback queues. */
-   
-  /* The socket server for the data client. */
-
-  GzochidSocketServer *socket_server;
-
-  /* The client's current connection to the data server. */
-
-  gzochid_client_socket *client_socket;
-
-  /* A buffer of bytes to be sent to the data server pending the availability of
-     a connected client socket. */
+  GMutex queue_mutex; /* Protects the callback queue table. */
   
-  GByteArray *outbound_messages;
+  /* A `GMainContext' for scheduling tasks (such as timed lock releases) to be 
+     run by the metaclient while connected to the server. This field is 
+     "inherited" from the metaclient. */
+
+  GMainContext *main_context; 
+
+  /* The reconnectable socket wrapping the connection to the metaserver, to be
+     used for writes. This field is "inherited" from the metaclient. */
   
-  char *hostname; /* Meta server hostname. */
-  unsigned int port; /* Meta server port. */
-
-  /* Interval between meta server connection attempts while in a disconnected 
-     state. */
-
-  guint connect_attempt_interval_seconds;
-  
-  GThread *thread; /* The connection maintenance thread. */
-  
-  /* Whether the connection maintenance thread should stay alive. */
-
-  gboolean running; 
-
-  GMutex mutex; /* Mutex protecting the client socket. */
-  GCond cond; /* Condition variable coordinating connection maintenance. */
+  gzochid_reconnectable_socket *socket;
 };
 
 /* Boilerplate setup for the data client object. */
@@ -135,7 +144,8 @@ G_DEFINE_TYPE (GzochidDataClient, gzochid_data_client, G_TYPE_OBJECT);
 enum gzochid_data_client_properties
   {
     PROP_CONFIGURATION = 1,
-    PROP_SOCKET_SERVER,
+    PROP_MAIN_CONTEXT,
+    PROP_SOCKET,
     N_PROPERTIES
   };
 
@@ -150,33 +160,52 @@ gzochid_data_client_set_property (GObject *object, guint property_id,
   switch (property_id)
     {
     case PROP_CONFIGURATION:
-      self->configuration = gzochid_configuration_extract_group
-	(GZOCHID_CONFIGURATION (g_value_get_object (value)), "metaserver");
-      break;
-      
-    case PROP_SOCKET_SERVER:
-      self->socket_server = g_object_ref (g_value_get_object (value));
+      self->configuration = g_object_ref (g_value_get_object (value));
       break;
 
+    case PROP_MAIN_CONTEXT:
+      self->main_context = g_main_context_ref (g_value_get_boxed (value));
+      break;
+
+    case PROP_SOCKET:
+      self->socket = g_value_get_pointer (value);
+      break;
+      
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
     }
 }
 
+static void
+gzochid_data_client_constructed (GObject *gobject)
+{
+  GzochidDataClient *client = GZOCHID_DATA_CLIENT (gobject);  
+
+  /* Extract the "metaserver" configuration group to use to look up lock 
+     release timeouts . */
+
+  GHashTable *metaserver_config = gzochid_configuration_extract_group
+    (client->configuration, "metaserver");
+  
+  client->lock_release_ms = gzochid_config_to_int
+    (g_hash_table_lookup (metaserver_config, "lock.release.msec"), 1000);
+  client->range_lock_release_ms = gzochid_config_to_int
+    (g_hash_table_lookup (metaserver_config, "rangelock.release.msec"), 500);
+
+  g_hash_table_destroy (metaserver_config);
+}
+
 static void
 gzochid_data_client_finalize (GObject *gobject)
 {
   GzochidDataClient *client = GZOCHID_DATA_CLIENT (gobject);
-
-  g_hash_table_destroy (client->configuration);
+  
   g_hash_table_destroy (client->application_callback_queues);
 
-  g_byte_array_unref (client->outbound_messages);
-
   g_mutex_clear (&client->queue_mutex);
-  g_mutex_clear (&client->mutex);
-  g_cond_clear (&client->cond);
+
+  G_OBJECT_CLASS (gzochid_data_client_parent_class)->finalize (gobject);  
 }
 
 static void
@@ -184,7 +213,9 @@ gzochid_data_client_dispose (GObject *gobject)
 {
   GzochidDataClient *client = GZOCHID_DATA_CLIENT (gobject);
 
-  g_object_unref (client->socket_server);
+  g_object_unref (client->configuration);
+
+  G_OBJECT_CLASS (gzochid_data_client_parent_class)->dispose (gobject);
 }
 
 static void
@@ -192,33 +223,39 @@ gzochid_data_client_class_init (GzochidDataClientClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->constructed = gzochid_data_client_constructed;
   object_class->dispose = gzochid_data_client_dispose;
   object_class->finalize = gzochid_data_client_finalize;
   object_class->set_property = gzochid_data_client_set_property;
 
   obj_properties[PROP_CONFIGURATION] = g_param_spec_object
-    ("configuration", "gzochi data client configuration",
-     "Set the gzochi server configuration", GZOCHID_TYPE_CONFIGURATION,
-     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT | G_PARAM_PRIVATE);
+    ("configuration", "config", "The global configuration object",
+     GZOCHID_TYPE_CONFIGURATION, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT);
 
-  obj_properties[PROP_SOCKET_SERVER] = g_param_spec_object
-    ("socket-server", "Socket server", "Set the socket server",
-     GZOCHID_TYPE_SOCKET_SERVER,
-     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT | G_PARAM_PRIVATE);
+  obj_properties[PROP_MAIN_CONTEXT] = g_param_spec_boxed
+    ("main-context", "main-context", "The meta client's main context",
+     G_TYPE_MAIN_CONTEXT, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT);
+
+  obj_properties[PROP_SOCKET] = g_param_spec_pointer
+    ("reconnectable-socket", "socket",
+     "The meta client's reconnectable socket",
+     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT);
 
   g_object_class_install_properties
     (object_class, N_PROPERTIES, obj_properties);
 }
 
 /* Create and return a new callback registration object with the specified
-   expected opcode and success and failure callback (with associated user data
-   pointers. This object should freed via `free' when no longer needed. */
+   expected opcode and success, failure, and release callbacks (with associated
+   user data pointers. This object should freed via `free' when no longer 
+   needed. */
 
 static dataclient_callback_registration *
 create_callback
 (unsigned char expected_opcode,
  gzochid_dataclient_success_callback success_callback, gpointer success_data,
- gzochid_dataclient_failure_callback failure_callback, gpointer failure_data)
+ gzochid_dataclient_failure_callback failure_callback, gpointer failure_data,
+ gzochid_dataclient_release_callback release_callback, gpointer release_data)
 {
   dataclient_callback_registration *registration =
     malloc (sizeof (dataclient_callback_registration));
@@ -231,6 +268,14 @@ create_callback
   registration->failure_callback = failure_callback;
   registration->failure_data = failure_data;
 
+  /* This initial release timeout is a sentinel value of sorts indicating that
+     the lock has not yet be acquired. */
+  
+  registration->timeout = G_MAXINT64;
+  
+  registration->release_callback = release_callback;
+  registration->release_data = release_data;
+  
   return registration;
 }
 
@@ -294,286 +339,41 @@ release_callback_queue (dataclient_callback_queue *queue)
 static void
 gzochid_data_client_init (GzochidDataClient *self)
 {
+  g_mutex_init (&self->queue_mutex);
   self->application_callback_queues = g_hash_table_new_full
     (g_str_hash, g_str_equal, free, (GDestroyNotify) free_callback_queue);
-
-  self->outbound_messages = g_byte_array_new ();
-
-  self->running = FALSE;
-  g_mutex_init (&self->queue_mutex);
-  g_mutex_init (&self->mutex);
-  g_cond_init (&self->cond);
+  
 }
 
 /* End boilerplate. */
 
-/* Attempts to connect to the specified hostname and port. Returns a new 
-   `gzochid_client_socket' on success, `NULL' on failure (in which case the
-   specified `GError' will be set, if specified). */
+/*
+  A convenience function for prefixing a message payload with its size and
+  opcode. Every message needs these things and the memory management is a
+  bit tedious. 
 
-static gzochid_client_socket *
-attempt_connect (GzochidDataClient *client, char *hostname, unsigned int port,
-		 GError **err)
+  Returns a pointer to a newly-allocated buffer containing the complete message,
+  which should be freed when no longer needed. The `formatted_len' argument, if
+  provided, will be set to the length of this buffer. 
+*/
+
+static unsigned char *
+format_message (guchar opcode, GBytes *payload, size_t *formatted_len)
 {
-  int sock;
-  GIOChannel *channel = NULL;
-  struct sockaddr_in name;
-  struct hostent *hostinfo = NULL;
-  char *connection_description = NULL;
-  gzochid_client_socket *client_socket = NULL;
-  int flag = 1;
+  size_t len = 0;
+  const unsigned char *data = g_bytes_get_data (payload, &len);
+  unsigned char *buf = malloc (sizeof (unsigned char) * (len + 3));
 
-  /* Create the client socket. */
+  /* Prefix the message with its length. */
   
-  sock = socket (PF_INET, SOCK_STREAM, 0);
-  if (sock < 0)
-    {
-      g_set_error
-	(err, GZOCHID_DATA_CLIENT_ERROR, GZOCHID_DATA_CLIENT_ERROR_SOCKET,
-	 "Couldn't create socket: %s", strerror (errno));
-      
-      return NULL;
-    }
+  gzochi_common_io_write_short (len, buf, 0);
+  buf[2] = opcode;
+  memcpy (buf + 3, data, len);
 
-  /* Resolve the meta server's address. */
+  if (formatted_len != NULL)
+    *formatted_len = len + 3;
   
-  name.sin_family = AF_INET;
-  name.sin_port = htons (port);
-  hostinfo = gethostbyname (hostname);
-
-  if (hostinfo == NULL)
-    {
-      g_set_error
-	(err, GZOCHID_DATA_CLIENT_ERROR, GZOCHID_DATA_CLIENT_ERROR_NETWORK,
-	 "Couldn't resolve %s: %s", hostname, strerror (errno));
-      
-      return NULL;
-    }
-
-  name.sin_addr = *(struct in_addr *) hostinfo->h_addr;
-
-  /* Create the connection. */
-  
-  if (connect
-      (sock, (struct sockaddr *) &name, sizeof (struct sockaddr_in)) < 0)
-    {
-      g_set_error
-	(err, GZOCHID_DATA_CLIENT_ERROR, GZOCHID_DATA_CLIENT_ERROR_NETWORK,
-	 "Couldn't connect to %s:%d: %s", hostname, port, strerror (errno));
-
-      return NULL;
-    }
-
-  setsockopt (sock, IPPROTO_TCP, TCP_NODELAY, (char *) &flag, sizeof (int));
-#if defined (__APPLE__) && defined (__MACH__)
-  setsockopt (sock, SOL_SOCKET, SO_NOSIGPIPE, (char *) &flag, sizeof (int));
-#endif
-
-  connection_description = g_strdup_printf ("%s:%d", hostname, port);
-
-  channel = g_io_channel_unix_new (sock);
-  
-  g_io_channel_set_encoding (channel, NULL, NULL);
-  g_io_channel_set_flags (channel, G_IO_FLAG_NONBLOCK, NULL);
-  g_io_channel_set_buffered (channel, FALSE);
-  
-  client_socket = gzochid_client_socket_new
-    (channel, connection_description, gzochid_dataclient_client_protocol,
-     client);
-
-  g_free (connection_description);
-
-  /* Add the client socket to the socket server's main loop. */
-  
-  gzochid_client_socket_listen (client->socket_server, client_socket);
-
-  return client_socket;
-}
-
-/* The body of the connection maintenance thread. The algorithm is, in English:
-   While there is no connection to the meta server, attempt to connect. If the
-   connection fails, wait a configured number of seconds before trying again; if
-   it succeeds, go to sleep and wait to be notified that the connection has 
-   died. */
-
-static gpointer
-ensure_connection (gpointer data)
-{
-  GzochidDataClient *client = data;
-
-  g_mutex_lock (&client->mutex);
-
-  while (client->running)
-    {
-      /* Is the socket disconnected? */
-      
-      while (client->client_socket == NULL)
-	{
-	  GError *err = NULL;
-	  gzochid_client_socket *socket = attempt_connect
-	    (client, client->hostname, client->port, &err);
-
-	  /* Did the connection attempt fail? */
-	  
-	  if (socket == NULL)
-	    {
-	      if (err != NULL)
-		{
-		  g_warning
-		    ("Failed to connect to meta server at %s:%d: %s; "
-		     "waiting %d seconds...", client->hostname, client->port,
-		     err->message, client->connect_attempt_interval_seconds);
-		  g_error_free (err);
-		}
-	      else g_warning
-		     ("Failed to connect to meta server at %s:%d; "
-		      "waiting %d seconds...", client->hostname, client->port,
-		      client->connect_attempt_interval_seconds);
-
-	      /* Sleep before trying again, for politeness. */
-	      
-	      g_usleep (client->connect_attempt_interval_seconds * 1000000);
-	    }
-	  else
-	    {
-	      client->client_socket = socket;
-	      g_message ("Connected to meta server at %s:%d.", client->hostname,
-			 client->port);
-
-	      if (client->outbound_messages->len > 0)
-		{
-		  gzochid_client_socket_write
-		    (client->client_socket, client->outbound_messages->data,
-		     client->outbound_messages->len);
-
-		  g_byte_array_set_size (client->outbound_messages, 0);
-		}
-	      
-	      /* Notify any waiting clients that the connection is open for
-		 business. */
-	      
-	      g_cond_broadcast (&client->cond);
-	    }
-	}
-
-      /* Wait for an event. */
-      
-      g_cond_wait (&client->cond, &client->mutex);
-    }
-
-  g_mutex_unlock (&client->mutex);  
-  return NULL;
-}
-
-/* Attempts to set the data ciient's meta server address from the address string
-   given in the "dataclient" section of the server configuration. Returns `TRUE'
-   if an address of the form "hostname:port" can be parsed out of the string,
-   `FALSE" otherwise. */
-
-static gboolean
-extract_hostname_port (GzochidDataClient *dataclient, GError **err)
-{
-  const char *hostname_port = g_hash_table_lookup
-    (dataclient->configuration, "server.address");
-
-  if (hostname_port == NULL)
-    {
-      dataclient->hostname = strdup ("localhost");
-      dataclient->port = 9001; /* The default port number. */
-
-      return TRUE;
-    }
-  else
-    {
-      GMatchInfo *match_info = NULL;
-      GRegex *address_regex = g_regex_new ("([^:]+):(\\d{1,5})", 0, 0, NULL);  
-      gboolean matched = g_regex_match
-	(address_regex, hostname_port, 0, &match_info);
-
-      g_regex_unref (address_regex);
-      
-      if (matched)
-	{
-	  gchar *port_str = g_match_info_fetch (match_info, 2);
-
-	  dataclient->hostname = g_match_info_fetch (match_info, 1);
-	  dataclient->port = atoi (port_str);
-
-	  g_free (port_str);
-	  g_match_info_free (match_info);
-	  
-	  return TRUE;
-	}
-      else
-	{
-	  g_match_info_free (match_info);
-	  
-	  g_set_error
-	    (err, GZOCHID_DATA_CLIENT_ERROR, GZOCHID_DATA_CLIENT_ERROR_ADDRESS,
-	     "Invalid meta server address: %s", hostname_port);
-
-	  return FALSE;
-	}
-    }
-}
-
-void
-gzochid_dataclient_start (GzochidDataClient *dataclient, GError **err)
-{
-  GError *local_err = NULL;
-
-  if (!extract_hostname_port (dataclient, &local_err))
-    {
-      if (local_err != NULL)
-	g_propagate_error (err, local_err);
-      return;
-    }
-
-  dataclient->connect_attempt_interval_seconds = gzochid_config_to_int
-    (g_hash_table_lookup (dataclient->configuration,
-			  "server.connect.interval.sec"), 5);
-
-  /* Set the running flag to keep the maintenance loop going. */
-  
-  dataclient->running = TRUE;
-  dataclient->thread = g_thread_new
-    ("dataclient-conn", ensure_connection, dataclient);
-}
-
-void
-gzochid_dataclient_stop (GzochidDataClient *dataclient)
-{
-  if (dataclient->thread != NULL)
-    {
-      g_free (dataclient->hostname);
-      dataclient->hostname = NULL;
-      dataclient->port = 0;
-      
-      g_mutex_lock (&dataclient->mutex);
-
-      /* Unset the running flag and wake up the maintenance thread so that it
-	 can exit. */
-      
-      dataclient->running = FALSE;
-      g_cond_signal (&dataclient->cond);
-
-      g_mutex_unlock (&dataclient->mutex);
-
-      /* Ensure that the maintenance thread has exited. */
-      
-      g_thread_join (dataclient->thread);
-    }
-}
-
-void
-gzochid_dataclient_nullify_connection (GzochidDataClient *dataclient)
-{
-  g_mutex_lock (&dataclient->mutex);
-
-  dataclient->client_socket = NULL;
-
-  g_cond_signal (&dataclient->cond);
-  g_mutex_unlock (&dataclient->mutex);
+  return buf;
 }
 
 /*
@@ -589,32 +389,9 @@ static void
 write_message (GzochidDataClient *client, guchar opcode, GBytes *payload)
 {
   size_t len = 0;
-  const unsigned char *data = g_bytes_get_data (payload, &len);
-  unsigned char *buf = malloc (sizeof (unsigned char) * (len + 3));
+  unsigned char *buf = format_message (opcode, payload, &len);
 
-  /* Prefix the message with its length. */
-  
-  gzochi_common_io_write_short (len, buf, 0);
-  buf[2] = opcode;
-  memcpy (buf + 3, data, len);
-
-  g_mutex_lock (&client->mutex);
-
-  
-  if (client->client_socket != NULL)
-
-    /* Queue up the bytes to sent on the socket. */
-    
-    gzochid_client_socket_write (client->client_socket, buf, len + 3);
-  else
-
-    /* If the client is known to be disconnected, queue up the message bytes to
-       be sent on reconnnect. */
-    
-    g_byte_array_append (client->outbound_messages, buf, len + 3);
-  
-  g_mutex_unlock (&client->mutex);
-  
+  gzochid_reconnectable_socket_write (client->socket, buf, len);
   free (buf);
 }
 
@@ -629,6 +406,10 @@ gzochid_dataclient_received_oids (GzochidDataClient *client,
   
   if (queue->oids_callback != NULL)
     {
+      gzochid_trace ("Received oid block { %" G_GUINT64_FORMAT
+		     "/%d }; invoking callback.", response->block.block_start,
+		     response->block.block_size);
+
       queue->oids_callback (response->block, queue->oids_callback_data);
 
       /* Null out the oids callback after receipt. */
@@ -636,6 +417,9 @@ gzochid_dataclient_received_oids (GzochidDataClient *client,
       queue->oids_callback = NULL;
       queue->oids_callback_data = NULL;
     }
+  else g_debug ("Received oid block { %" G_GUINT64_FORMAT
+		"/%d }, but no pending callbacks.", response->block.block_start,
+		response->block.block_size);
 
   release_callback_queue (queue);
 }
@@ -647,6 +431,8 @@ gzochid_dataclient_reserve_oids (GzochidDataClient *client, char *app,
 {
   GBytes *message_bytes = g_bytes_new (app, strlen (app) + 1);
   dataclient_callback_queue *queue = acquire_callback_queue (client, app);
+
+  gzochid_trace ("Requesting oid block from meta server.");
   
   write_message (client, GZOCHID_DATA_PROTOCOL_REQUEST_OIDS, message_bytes);
   g_bytes_unref (message_bytes);
@@ -661,6 +447,14 @@ gzochid_dataclient_reserve_oids (GzochidDataClient *client, char *app,
   release_callback_queue (queue);
 }
 
+static gboolean
+invoke_release_callback (gpointer user_data)
+{
+  dataclient_callback_registration *registration = user_data;
+  registration->release_callback (registration->release_data);  
+  return FALSE;
+}
+
 /* Convenience function to handle the processing of a message received in
    response to a value or sequential key request, and representing a successful
    or unsuccessful fulfillment of the request. Some error checking is performed
@@ -668,7 +462,8 @@ gzochid_dataclient_reserve_oids (GzochidDataClient *client, char *app,
    requests were submitted. */
 
 static void
-process_queued_callback (dataclient_callback_queue *queue, unsigned char opcode,
+process_queued_callback (GzochidDataClient *client,
+			 dataclient_callback_queue *queue, unsigned char opcode,
 			 gzochid_data_response *response)
 {
   GList *callback_link = queue->callback_registrations;
@@ -681,14 +476,49 @@ process_queued_callback (dataclient_callback_queue *queue, unsigned char opcode,
      callback registration at the head of the queue. */
   
   if (opcode != callbacks->expected_opcode)
-    g_warning
-      ("Received response %d for %s/%s; expected response %d.", opcode,
-       response->app, response->store, callbacks->expected_opcode);
+    {
+      g_warning
+	("Received response %d for %s/%s; expected response %d.", opcode,
+	 response->app, response->store, callbacks->expected_opcode);
+      free (callbacks);
+    }
   else if (response->success)
-    callbacks->success_callback (response->data, callbacks->success_data);
-  else callbacks->failure_callback (response->timeout, callbacks->failure_data);
+    {
+      GSource *release_callback = NULL;
+      
+      callbacks->success_callback (response->data, callbacks->success_data);
+      assert (callbacks->timeout == G_MAXINT64);
 
-  free (callbacks);      
+      if (opcode == GZOCHID_DATA_PROTOCOL_VALUE_RESPONSE)
+	{
+	  gzochid_trace
+	    ("Obtained lock for %s/%s; will expire in %dms.",
+	     response->app, response->store, client->lock_release_ms);
+	  release_callback = g_timeout_source_new (client->lock_release_ms);
+	}
+      else if (opcode == GZOCHID_DATA_PROTOCOL_NEXT_KEY_RESPONSE)
+	{
+	  gzochid_trace
+	    ("Obtained range lock on keys in %s/%s; will expired in %dms.",
+	     response->app, response->store, client->range_lock_release_ms);
+	  release_callback = g_timeout_source_new
+	    (client->range_lock_release_ms);
+	}
+
+      if (release_callback != NULL)
+	{
+	  g_source_set_callback
+	    (release_callback, invoke_release_callback, callbacks, free);
+	  g_source_attach (release_callback, client->main_context);
+	}
+    }
+  else
+    {
+      g_debug
+	("Lock request against %s/%s failed.", response->app, response->store);
+      callbacks->failure_callback (response->timeout, callbacks->failure_data);
+      free (callbacks);
+    }
 }
 
 void
@@ -703,7 +533,7 @@ gzochid_dataclient_received_value (GzochidDataClient *client,
       ("Received value for %s/%s but no callbacks registered.", response->app,
        response->store);
   else process_queued_callback
-	 (queue, GZOCHID_DATA_PROTOCOL_VALUE_RESPONSE, response);
+	 (client, queue, GZOCHID_DATA_PROTOCOL_VALUE_RESPONSE, response);
 
   release_callback_queue (queue);
 }
@@ -711,9 +541,10 @@ gzochid_dataclient_received_value (GzochidDataClient *client,
 void
 gzochid_dataclient_request_value
 (GzochidDataClient *client, char *app, char *store, GBytes *key,
- gboolean for_write, gzochid_dataclient_success_callback success_callback,
- gpointer success_data, gzochid_dataclient_failure_callback failure_callback,
- gpointer failure_data)
+ gboolean for_write,
+ gzochid_dataclient_success_callback success_callback, gpointer success_data,
+ gzochid_dataclient_failure_callback failure_callback, gpointer failure_data,
+ gzochid_dataclient_release_callback release_callback, gpointer release_data)
 {
   dataclient_callback_queue *queue = acquire_callback_queue (client, app);
   GByteArray *payload = g_byte_array_new ();
@@ -742,6 +573,12 @@ gzochid_dataclient_request_value
   g_byte_array_append (payload, key_bytes, key_len);
   
   payload_bytes = g_byte_array_free_to_bytes (payload);
+
+  if (gzochid_log_level_visible (G_LOG_DOMAIN, GZOCHID_LOG_LEVEL_TRACE))
+    GZOCHID_WITH_FORMATTED_BYTES
+      (key, buf, 33, gzochid_trace ("Requesting %s lock on %s/%s/%s.",
+				    LOCK_ACCESS (for_write), app, store, buf));
+
   write_message (client, GZOCHID_DATA_PROTOCOL_REQUEST_VALUE, payload_bytes);
   g_bytes_unref (payload_bytes);
 
@@ -751,7 +588,8 @@ gzochid_dataclient_request_value
     (queue->callback_registrations,
      create_callback (GZOCHID_DATA_PROTOCOL_VALUE_RESPONSE,
 		      success_callback, success_data,
-		      failure_callback, failure_data));
+		      failure_callback, failure_data,
+		      release_callback, release_data));
 
   release_callback_queue (queue);
 }
@@ -768,26 +606,18 @@ gzochid_dataclient_received_next_key (GzochidDataClient *client,
       ("Received key range response for %s/%s but no callbacks registered.",
        response->app, response->store);
   else process_queued_callback
-	 (queue, GZOCHID_DATA_PROTOCOL_NEXT_KEY_RESPONSE, response);
+	 (client, queue, GZOCHID_DATA_PROTOCOL_NEXT_KEY_RESPONSE, response);
 
   release_callback_queue (queue);
 }
 
-void
-gzochid_dataclient_request_next_key
-(GzochidDataClient *client, char *app, char *store, GBytes *key,
- gzochid_dataclient_success_callback success_callback, gpointer success_data,
- gzochid_dataclient_failure_callback failure_callback, gpointer failure_data)
+/* Convenience function to append a length-prefixed array of bytes to the
+   specified `GByteArray', or a pair of `NULL' bytes if the specified byte
+   buffer is `NULL'. */
+
+static void
+write_nullable_bytes (GByteArray *payload, GBytes *key)
 {
-  dataclient_callback_queue *queue = acquire_callback_queue (client, app);
-  GByteArray *payload = g_byte_array_new ();
-  GBytes *payload_bytes = NULL;
-  
-  /* Serialize the key request message. */
-
-  g_byte_array_append (payload, (unsigned char *) app, strlen (app) + 1);
-  g_byte_array_append (payload, (unsigned char *) store, strlen (store) + 1);
-
   if (key != NULL)
     {
       size_t payload_len = 0, key_len = 0;
@@ -806,8 +636,38 @@ gzochid_dataclient_request_next_key
     }
   else g_byte_array_append
 	 (payload, (unsigned char *) &(unsigned char[]) { 0, 0 }, 2);
+}
+
+void
+gzochid_dataclient_request_next_key
+(GzochidDataClient *client, char *app, char *store, GBytes *key,
+ gzochid_dataclient_success_callback success_callback, gpointer success_data,
+ gzochid_dataclient_failure_callback failure_callback, gpointer failure_data,
+ gzochid_dataclient_release_callback release_callback, gpointer release_data)
+{
+  dataclient_callback_queue *queue = acquire_callback_queue (client, app);
+  GByteArray *payload = g_byte_array_new ();
+  GBytes *payload_bytes = NULL;
+  
+  /* Serialize the key request message. */
+
+  g_byte_array_append (payload, (unsigned char *) app, strlen (app) + 1);
+  g_byte_array_append (payload, (unsigned char *) store, strlen (store) + 1);
+
+  write_nullable_bytes (payload, key);
 
   payload_bytes = g_byte_array_free_to_bytes (payload);
+
+  if (gzochid_log_level_visible (G_LOG_DOMAIN, GZOCHID_LOG_LEVEL_TRACE))
+    {
+      if (key != NULL)
+	GZOCHID_WITH_FORMATTED_BYTES
+	  (key, buf, 33, gzochid_trace
+	   ("Requesting range lock starting at %s/%s/%s.", app, store, buf));
+      else gzochid_trace
+	     ("Requesting range lock on %s/%s keyspace.", app, store);
+    }
+
   write_message (client, GZOCHID_DATA_PROTOCOL_REQUEST_NEXT_KEY, payload_bytes);
   g_bytes_unref (payload_bytes);
   
@@ -817,7 +677,8 @@ gzochid_dataclient_request_next_key
     (queue->callback_registrations,
      create_callback (GZOCHID_DATA_PROTOCOL_NEXT_KEY_RESPONSE,
 		      success_callback, success_data,
-		      failure_callback, failure_data));
+		      failure_callback, failure_data,
+		      release_callback, release_data));
 
   release_callback_queue (queue);
 }
@@ -828,19 +689,129 @@ gzochid_dataclient_submit_changeset
 {
   gzochid_data_changeset *changeset = gzochid_data_changeset_new (app, changes);
   GByteArray *payload = g_byte_array_new ();
-  GBytes *payload_bytes = NULL;
 
   /* Serialize the changeset submission message. */
   
   gzochid_data_protocol_changeset_write (changeset, payload);
 
-  payload_bytes = g_byte_array_free_to_bytes (payload); 
-  write_message (client, GZOCHID_DATA_PROTOCOL_SUBMIT_CHANGESET, payload_bytes);
-  g_bytes_unref (payload_bytes);
+  gzochid_trace
+    ("Submitting changeset for %s with %d elements; total %d bytes.", app,
+     changes->len, payload->len);
+  
+  if (payload->len <= MAX_MESSAGE_LENGTH)
+    {
+      GBytes *payload_bytes = g_byte_array_free_to_bytes (payload);
 
+      write_message
+	(client, GZOCHID_DATA_PROTOCOL_SUBMIT_CHANGESET, payload_bytes);
+      g_bytes_unref (payload_bytes);
+    }
+  else
+    {
+      g_critical
+	("Refusing to submit %d-byte changeset for %s to meta server.",
+	 payload->len, app);
+      g_byte_array_unref (payload);
+    }
+  
   gzochid_data_changeset_free (changeset);
 }
 
+void gzochid_dataclient_release_key
+(GzochidDataClient *client, char *app, char *store, GBytes *key)
+{
+  GByteArray *payload = g_byte_array_new ();
+  const unsigned char *key_bytes = NULL;
+  size_t payload_len = 0, key_len = 0, len;
+  GBytes *payload_bytes = NULL;
+  unsigned char *buf = NULL;
+  
+  /* Serialize the key request message. */
+
+  g_byte_array_append (payload, (unsigned char *) app, strlen (app) + 1);
+  g_byte_array_append (payload, (unsigned char *) store, strlen (store) + 1);
+
+  key_bytes = g_bytes_get_data (key, &key_len);
+
+  /* Grow the byte array by 2 bytes. */
+  
+  payload_len = payload->len;
+  g_byte_array_set_size (payload, payload_len + 2);
+
+  /* Write the key length directly to the buffer. */
+  
+  gzochi_common_io_write_short (key_len, payload->data, payload_len);
+  
+  g_byte_array_append (payload, key_bytes, key_len);
+  
+  payload_bytes = g_byte_array_free_to_bytes (payload);
+
+  /* We can't call `write_message' here because it wants to lock the client
+     mutex. */
+
+  buf = format_message (GZOCHID_DATA_PROTOCOL_RELEASE_KEY, payload_bytes, &len);
+
+  if (gzochid_log_level_visible (G_LOG_DOMAIN, GZOCHID_LOG_LEVEL_TRACE))
+    GZOCHID_WITH_FORMATTED_BYTES
+      (key, fmt, 33, gzochid_trace
+       ("Releasing key %s/%s/%s.", app, store, fmt));
+
+  gzochid_reconnectable_socket_write (client->socket, buf, len);
+  free (buf);
+
+  g_bytes_unref (payload_bytes);
+}
+
+void gzochid_dataclient_release_key_range
+(GzochidDataClient *client, char *app, char *store, GBytes *from, GBytes *to)
+{
+  GByteArray *payload = g_byte_array_new ();
+  GBytes *payload_bytes = NULL;
+  unsigned char *buf = NULL;
+  size_t len = 0;
+  
+  /* Serialize the key request message. */
+
+  g_byte_array_append (payload, (unsigned char *) app, strlen (app) + 1);
+  g_byte_array_append (payload, (unsigned char *) store, strlen (store) + 1);
+
+  write_nullable_bytes (payload, from);   
+  write_nullable_bytes (payload, to);
+
+  payload_bytes = g_byte_array_free_to_bytes (payload);
+
+  /* We can't call `write_message' here because it wants to lock the client
+     mutex. */
+  
+  buf = format_message
+    (GZOCHID_DATA_PROTOCOL_RELEASE_KEY_RANGE, payload_bytes, &len);
+
+  if (gzochid_log_level_visible (G_LOG_DOMAIN, GZOCHID_LOG_LEVEL_TRACE))
+    {
+      if (from != NULL)
+	GZOCHID_WITH_FORMATTED_BYTES
+	  (from, buf1, 33,
+	   if (to != NULL)
+	     GZOCHID_WITH_FORMATTED_BYTES
+	       (to, buf2, 33, gzochid_trace
+		("Releasing range lock on %s/%s from %s to %s.", app, store,
+		 buf1, buf2));
+	   else gzochid_trace ("Releasing range lock starting at %s/%s/%s.",
+			       app, store, buf1));
+      else if (to != NULL)
+	GZOCHID_WITH_FORMATTED_BYTES
+	  (to, buf2, 33, gzochid_trace
+	   ("Releasing range lock ending at %s/%s/%s.", app, store, buf2));
+      else gzochid_trace
+	     ("Releasing range lock on %s/%s keyspace.", app, store);
+    }
+
+  gzochid_reconnectable_socket_write (client->socket, buf, len);
+  free (buf);
+
+  g_bytes_unref (payload_bytes);
+}
+
 GQuark
 gzochid_data_client_error_quark ()
 {
