@@ -1,5 +1,5 @@
 /* durable-task.c: Persistent task management routines for gzochid
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -24,9 +24,9 @@
 
 #include "app.h"
 #include "auth_int.h"
+#include "data.h"
 #include "durable-task.h"
 #include "gzochid-auth.h"
-#include "game.h"
 #include "txlog.h"
 #include "util.h"
 
@@ -78,11 +78,32 @@ create_transaction_context (gzochid_application_context *app_context)
   return tx_context;
 }
 
+/* Wrapper around `gzochid_task_free' to allow the embedded 
+   `gzochid_application_task' to be unref'd before the outer task is freed. (For
+   obvious reasons, this must only be used on tasks that wrap a 
+   `gzochid_application_task'!) */
+
+static void
+task_free_wrapper (gpointer data)
+{
+  gzochid_task *task = data;
+
+  gzochid_application_task_unref (task->data);
+  gzochid_task_free (task);
+}
+
+/* Releases the resources used by the specified task transaction. The `unref'
+   flag determines whether the application tasks are unref'd (in the case of
+   rollback, in which they will never be executed) or not (in the case of 
+   commit, in which they will be unref'd after execution). */ 
+
 static void 
-cleanup_transaction (gzochid_task_transaction_context *tx_context)
+cleanup_transaction (gzochid_task_transaction_context *tx_context,
+		     gboolean unref_app_tasks)
 {
   g_list_free_full
-    (tx_context->scheduled_tasks, (GDestroyNotify) gzochid_task_free);
+    (tx_context->scheduled_tasks,
+     unref_app_tasks ? task_free_wrapper : (GDestroyNotify) gzochid_task_free);
   free (tx_context);
 }
 
@@ -91,10 +112,8 @@ commit_scheduled_task (gpointer task, gpointer user_data)
 {
   gzochid_task_transaction_context *tx_context = user_data;
   gzochid_application_context *context = tx_context->context;
-  gzochid_game_context *game_context = (gzochid_game_context *)
-    ((gzochid_context *) context)->parent;
 
-  gzochid_schedule_submit_task (game_context->task_queue, task);
+  gzochid_schedule_submit_task (context->task_queue, task);
 }
 
 static void 
@@ -104,14 +123,13 @@ task_commit (gpointer data)
 
   g_list_foreach (tx_context->scheduled_tasks, commit_scheduled_task, data);
 
-  cleanup_transaction (tx_context);
+  cleanup_transaction (tx_context, FALSE);
 }
 
 static void 
 task_rollback (gpointer data)
 { 
-  gzochid_task_transaction_context *tx_context = data;
-  cleanup_transaction (tx_context);
+  cleanup_transaction (data, TRUE);
 }
 
 static gzochid_transaction_participant task_participant = 
@@ -355,7 +373,7 @@ static char *
 create_pending_task_binding (guint64 oid)
 {
   GString *str = g_string_new (PENDING_TASK_PREFIX);
-  g_string_printf (str, "%" G_GUINT64_FORMAT, oid);
+  g_string_append_printf (str, "%" G_GUINT64_FORMAT, oid);
   return g_string_free (str, FALSE);
 }
 
@@ -547,18 +565,17 @@ static void
 durable_task_cleanup_worker (gzochid_application_context *context,
 			     gzochid_auth_identity *identity, gpointer data)
 {
-  free (data);
+  g_free (data);
 }
 
 void 
 gzochid_schedule_durable_task 
 (gzochid_application_context *context, gzochid_auth_identity *identity,
- gzochid_application_task *task, 
- gzochid_application_task_serialization *serialization)
+ gzochid_application_task *task,
+ gzochid_application_task_serialization *serialization, GError **err)
 {
-  struct timeval immediate = { 0, 0 };
   gzochid_schedule_delayed_durable_task 
-    (context, identity, task, serialization, immediate);
+    (context, identity, task, serialization, (struct timeval) { 0, 0 }, err);
 }
 
 void 
@@ -566,14 +583,17 @@ gzochid_schedule_delayed_durable_task
 (gzochid_application_context *context, gzochid_auth_identity *identity,
  gzochid_application_task *task, 
  gzochid_application_task_serialization *serialization, 
- struct timeval delay)
+ struct timeval delay, GError **err)
 {
+  GError *local_err = NULL;
+  
   gzochid_durable_application_task_handle *handle = 
     gzochid_create_durable_application_task_handle
-    (task, serialization, delay, NULL, NULL);
+    (task, serialization, delay, NULL, &local_err);
 
   if (handle != NULL)
-    gzochid_schedule_durable_task_handle (context, handle, NULL);
+    gzochid_schedule_durable_task_handle (context, handle, err);
+  else g_propagate_error (err, local_err);
 }
 
 gzochid_periodic_task_handle *
@@ -581,31 +601,34 @@ gzochid_schedule_periodic_durable_task
 (gzochid_application_context *context, gzochid_auth_identity *identity, 
  gzochid_application_task *task,
  gzochid_application_task_serialization *serialization, 
- struct timeval delay, struct timeval period)
+ struct timeval delay, struct timeval period, GError **err)
 {
+  GError *local_err = NULL;
   gzochid_durable_application_task_handle *handle = 
     gzochid_create_durable_application_task_handle
-    (task, serialization, delay, &period, NULL);
+    (task, serialization, delay, &period, &local_err);
 
   if (handle != NULL)
     {
-      GError *err = NULL;
+      gzochid_schedule_durable_task_handle (context, handle, &local_err);
 
-      gzochid_schedule_durable_task_handle (context, handle, &err);
-
-      if (err == NULL)
+      if (local_err == NULL)
 	return handle;
       else
 	{
 	  g_debug
 	    ("Failed to schedule durable task in transaction: %s",
-	     err->message);
+	     local_err->message);
 
-	  g_error_free (err);
+	  g_propagate_error (err, local_err);
 	  return NULL;
 	}
     }
-  else return NULL;
+  else
+    {
+      g_propagate_error (err, local_err);
+      return NULL;
+    }
 }
 
 void gzochid_schedule_durable_task_handle
@@ -615,10 +638,6 @@ void gzochid_schedule_durable_task_handle
   guint64 *oid = NULL;
   GError *local_err = NULL;
 
-  gzochid_game_context *game_context = (gzochid_game_context *) 
-    ((gzochid_context *) app_context)->parent;
-  gzochid_auth_identity *cloned_identity = NULL;
-
   gzochid_application_task *transactional_task = NULL;
   gzochid_application_task *catch_task = NULL;
   gzochid_application_task *cleanup_task = NULL;
@@ -646,9 +665,8 @@ void gzochid_schedule_durable_task_handle
     (durable_task_handle_reference->oid);
 
   /* The memory allocated for this oid is freed by the cleanup task. */
-
-  oid = malloc (sizeof (guint64));
-  *oid = durable_task_handle_reference->oid;
+  
+  oid = g_memdup (&durable_task_handle_reference->oid, sizeof (guint64));
 
   task_handle->binding = create_pending_task_binding
     (durable_task_handle_reference->oid);
@@ -659,7 +677,7 @@ void gzochid_schedule_durable_task_handle
 
   if (local_err != NULL)
     {
-      free (oid);
+      g_free (oid);
       gzochid_durable_application_task_free (durable_task);      
 
       g_propagate_error (err, local_err);
@@ -669,17 +687,15 @@ void gzochid_schedule_durable_task_handle
   
   tx_context = join_transaction (app_context);
   
-  cloned_identity = gzochid_auth_identity_ref (task_handle->identity);
-
   transactional_task = gzochid_application_task_new
-    (app_context, cloned_identity, durable_task_application_worker,
+    (app_context, task_handle->identity, durable_task_application_worker,
      durable_task);
   catch_task = gzochid_application_task_new
-    (app_context, cloned_identity, durable_task_catch_worker, oid);
+    (app_context, task_handle->identity, durable_task_catch_worker, oid);
   cleanup_task = gzochid_application_task_new
-    (app_context, cloned_identity, durable_task_cleanup_worker, oid);
+    (app_context, task_handle->identity, durable_task_cleanup_worker, oid);
   execution = gzochid_transactional_application_task_timed_execution_new 
-    (transactional_task, catch_task, cleanup_task, game_context->tx_timeout);
+    (transactional_task, catch_task, cleanup_task, app_context->tx_timeout);
 
   /* Not necessary to hold a ref to these, as we've transferred them to the
      execution. */
@@ -689,7 +705,7 @@ void gzochid_schedule_durable_task_handle
   gzochid_application_task_unref (cleanup_task);
   
   application_task = gzochid_application_task_new
-    (app_context, cloned_identity,
+    (app_context, task_handle->identity,
      gzochid_application_resubmitting_transactional_task_worker, execution);
 
   tx_context->scheduled_tasks = g_list_append 
@@ -1022,7 +1038,7 @@ static void
 coordinator_cleanup_worker (gzochid_application_context *app_context,
 			    gzochid_auth_identity *identity, gpointer data)
 {
-  free (data);
+  g_free (data);
 }
 
 /* Enqueues a new instance of the coordinator task to be submitted to the
@@ -1040,9 +1056,6 @@ schedule_coordinator_task (gzochid_application_context *app_context,
   gzochid_task_transaction_context *tx_context =
     join_transaction (app_context);
 
-  gzochid_game_context *game_context = (gzochid_game_context *)
-    ((gzochid_context *) app_context)->parent;
-
   gzochid_data_managed_reference *chain_context_ref =
     gzochid_data_create_reference
     (app_context, &task_chain_context_serialization, chain_context, &err);
@@ -1063,8 +1076,7 @@ schedule_coordinator_task (gzochid_application_context *app_context,
       return;
     }
 
-  oid = malloc (sizeof (guint64));
-  *oid = chain_context_ref->oid;
+  oid = g_memdup (&chain_context_ref->oid, sizeof (guint64));
 
   /* Create the worker, catch, and cleanup tasks for the coordinator... */
   
@@ -1080,7 +1092,7 @@ schedule_coordinator_task (gzochid_application_context *app_context,
   coordinator_execution =
     gzochid_transactional_application_task_timed_execution_new
     (coordinator_main_task, coordinator_catch_task, coordinator_cleanup_task,
-     game_context->tx_timeout);
+     app_context->tx_timeout);
 
   /* Not necessary to hold a ref to these, as we've transferred them to the
      execution. */
