@@ -1,5 +1,5 @@
 /* httpd-app.h: Prototypes and declarations for httpd-app.c
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
 #include "game.h"
 #include "httpd.h"
+#include "resolver.h"
 
 /* Configure the specified HTTP server context with a RESTful hierarchy of 
    handlers for providing feedback about the state of a running gzochi 
@@ -34,10 +35,11 @@
          /oids/ - List of object ids
 	   [oid] - Object contents, in hex dump format.
 
-  The specified `gzochid_game_context' is used to resolve application name
-  references. */
+  The specified `GzochidGameServer' and `GzochidResolutionContext' are used
+  to resolve application name references and other parts of the container 
+  infrastructure. */
 
 void gzochid_httpd_app_register_handlers
-(GzochidHttpServer *, gzochid_game_context *);
+(GzochidHttpServer *, GzochidGameServer *, GzochidResolutionContext *);
 
 #endif /* GZOCHID_HTTPD_APP_H */
