@@ -1,5 +1,5 @@
 /* scheme-task.c: Scheme callback handlers and interface to gzochi Scheme API
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -165,8 +165,6 @@ gzochid_scheme_application_initialized_worker
 static gzochid_application_callback *
 scm_to_callback (gzochid_application_context *context, SCM scm_callback)
 {
-  GList *module = gzochid_scheme_callback_module (scm_callback);
-  char *procedure = gzochid_scheme_callback_procedure (scm_callback);
   gzochid_scm_location_info *scm_callback_reloc = gzochid_scm_location_get
     (context, scm_callback);
   
@@ -176,8 +174,18 @@ scm_to_callback (gzochid_application_context *context, SCM scm_callback)
 
   if (reference == NULL) /* Can happen on transaction timeout. */
     return NULL;
-  else return gzochid_application_callback_new
-	 (procedure, module, reference->oid);
+  else
+    {
+      GList *module = gzochid_scheme_callback_module (scm_callback);
+      char *procedure = gzochid_scheme_callback_procedure (scm_callback);
+      gzochid_application_callback *callback = gzochid_application_callback_new
+	(procedure, module, reference->oid);
+
+      g_list_free_full (module, free);      
+      free (procedure);
+      
+      return callback;
+    }
 }
 
 static gpointer 
@@ -272,6 +280,7 @@ gzochid_scheme_application_logged_in_worker
     }
   else
     {
+      gzochid_client_session_handler *session_handler = NULL;
       gzochid_scm_location_info *scm_handler_reloc = 
 	gzochid_scm_location_get (context, handler);
       gzochid_data_managed_reference *handler_reference = 
@@ -279,13 +288,21 @@ gzochid_scheme_application_logged_in_worker
 	(context, &gzochid_scm_location_aware_serialization, scm_handler_reloc,
 	 NULL);
 
-      scm_gc_protect_object (handler);
+      /* If the reference couldn't be created, bail out. */
       
-      gzochid_client_session_set_handler
-	(session, gzochid_with_application_context 
-	 (context, identity, unpack_handler, handler));
-      gzochid_client_session_set_handler_scm_oid
-	(session, handler_reference->oid);
+      if (handler_reference == NULL)
+	return;
+      
+      scm_gc_protect_object (handler);
+
+      session_handler = gzochid_with_application_context 
+	(context, identity, unpack_handler, handler);
+
+      /* Important to mark the session before setting the handlers on it. If the
+	 mark fails, the code below will attempt to clean up the handlers
+	 explicitly. But the data manager already knows about the session and
+	 the associated finalizer will attempt to clean up any non-`NULL' 
+	 handlers it finds during finalizaiton. */
       
       gzochid_data_mark 
 	(context, &gzochid_client_session_serialization, session, &err);
@@ -299,6 +316,12 @@ gzochid_scheme_application_logged_in_worker
 	    (context, &gzochid_scm_location_aware_serialization, 
 	     scm_session_reloc, &err);
 
+	  /* Now it's safe to install the handlers. */
+	  
+	  gzochid_client_session_set_handler (session, session_handler);
+	  gzochid_client_session_set_handler_scm_oid
+	    (session, handler_reference->oid);
+
 	  if (err == NULL)
 	    {
 	      gzochid_client_session_set_scm_oid
@@ -306,6 +329,15 @@ gzochid_scheme_application_logged_in_worker
 	      gzochid_client_session_send_login_success (context, session);
 	    }
 	}
+      else
+	{
+	  if (session_handler->received_message != NULL)
+	    gzochid_application_callback_free
+	      (session_handler->received_message);
+	  if (session_handler->disconnected != NULL)
+	    gzochid_application_callback_free (session_handler->disconnected);
+	  free (session_handler);
+	}
       
       if (err != NULL)
 	g_error_free (err);
@@ -501,7 +533,7 @@ gzochid_scheme_application_disconnected_cleanup_worker
 (gzochid_application_context *context, gzochid_auth_identity *identity, 
  gpointer ptr)
 {
-  free (ptr);
+  g_free (ptr);
 }
 
 void
