@@ -1,5 +1,5 @@
 /* socket.c: Application socket server implementation for gzochid
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -33,12 +33,14 @@ G_DEFINE_TYPE (GzochidSocketServer, gzochid_socket_server, G_TYPE_OBJECT);
 /* Lifecycle functions for the `GzochidSocketServer' object. */
 
 static void
-gzochid_socket_server_dispose (GObject *gobject)
+gzochid_socket_server_finalize (GObject *gobject)
 {
   GzochidSocketServer *server = GZOCHID_SOCKET_SERVER (gobject);
 
   g_main_context_unref (server->main_context);
   g_main_loop_unref (server->main_loop);
+
+  G_OBJECT_CLASS (gzochid_socket_server_parent_class)->finalize (gobject);
 }
 
 static void
@@ -46,7 +48,7 @@ gzochid_socket_server_class_init (GzochidSocketServerClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  object_class->dispose = gzochid_socket_server_dispose;
+  object_class->finalize = gzochid_socket_server_finalize;
 }
 
 static void
@@ -85,6 +87,8 @@ struct _gzochid_client_socket
 
   gzochid_client_protocol protocol; /* The client protocol. */
   gpointer protocol_data; /* Callback data for the client protocol. */
+
+  gint ref_count; /* The socket's reference count. */
 };
 
 gzochid_server_socket *
@@ -140,6 +144,8 @@ dispatch_client_write (GIOChannel *channel, GIOCondition cond, gpointer data)
   if (sock->send_buffer->len == 0)
     {
       g_source_destroy (sock->write_source);
+      g_source_unref (sock->write_source);
+
       sock->write_source = NULL;
     }
 
@@ -150,7 +156,8 @@ dispatch_client_write (GIOChannel *channel, GIOCondition cond, gpointer data)
 static gboolean
 dispatch_client_error (gzochid_client_socket *sock)
 {
-  g_debug ("Socket disconnected.");
+  g_debug ("Socket %s disconnected.",
+	   gzochid_client_socket_get_connection_description (sock));
   sock->protocol.error (sock->protocol_data);
   return FALSE;
 }
@@ -174,6 +181,8 @@ fill_recv_buffer (gzochid_client_socket *sock)
 
       if (error != NULL)
 	{
+	  g_warning ("Encountered error while reading from socket: %s",
+		     error->message);
 	  g_error_free (error);
 	  return FALSE;
 	}
@@ -208,6 +217,13 @@ dispatch_client (GIOChannel *channel, GIOCondition condition, gpointer data)
 {
   gzochid_client_socket *sock = data;
 
+  /* It's possible that a dangling read or error may be dispatched to a 
+     destroyed source, which may already have been finalized. So check the 
+     destroyed status before doing anything else. */
+  
+  if (g_source_is_destroyed (g_main_current_source ()))
+    return FALSE;
+
   switch (condition)
     {
     case G_IO_IN:
@@ -225,42 +241,26 @@ gzochid_client_socket_new (GIOChannel *channel,
 			   gpointer protocol_data)
 {
   gzochid_client_socket *sock = malloc (sizeof (gzochid_client_socket));
-  
+
+  sock->server = NULL;
   sock->channel = channel;
   sock->connection_description = strdup (connection_description);
 
   sock->protocol = protocol;
   sock->protocol_data = protocol_data;
   
+  sock->read_source = NULL;
   sock->write_source = NULL;
 
   g_mutex_init (&sock->sock_mutex);
   sock->recv_buffer = g_byte_array_new ();
   sock->send_buffer = g_byte_array_new ();
 
+  sock->ref_count = 1;
+  
   return sock;
 }
 
-/* Free the client socket and all of its non-GSource members. (Those get freed
-   asynchronously via the source destroy functions in 
-   `gzochid_client_socket_free'. */
-
-static void
-free_client_socket (gpointer data)
-{
-  gzochid_client_socket *sock = data;
-
-  sock->protocol.free (sock->protocol_data);
-
-  g_object_unref (sock->server);
-  g_io_channel_unref (sock->channel);
-  g_mutex_clear (&sock->sock_mutex);
-  g_byte_array_unref (sock->recv_buffer);
-  g_byte_array_unref (sock->send_buffer);
-  g_free (sock->connection_description);
-  free (sock);
-}
-
 void gzochid_server_socket_free (gzochid_server_socket *sock)
 {
   free (sock->name);
@@ -272,7 +272,6 @@ free_server_socket (gpointer data)
 {
   gzochid_server_socket *sock = data;
 
-  g_object_unref (sock->server);  
   g_io_channel_unref (sock->channel);
   free (sock->addr);
   
@@ -292,11 +291,12 @@ add_client (GzochidSocketServer *server, gzochid_client_socket *sock)
      destroyed. */
   
   g_source_set_callback
-    (sock->read_source, (GSourceFunc) dispatch_client, sock,
-     free_client_socket);
+    (sock->read_source, (GSourceFunc) dispatch_client,
+     gzochid_client_socket_ref (sock),
+     (GDestroyNotify) gzochid_client_socket_unref);
   g_source_attach (sock->read_source, server->main_context);
   
-  sock->server = g_object_ref (server);  
+  sock->server = server;  
 }
 
 static gboolean
@@ -332,6 +332,7 @@ dispatch_accept (GIOChannel *channel, GIOCondition cond, gpointer data)
       g_io_channel_set_buffered (channel, FALSE);
 
       add_client (server_socket->server, sock);
+      gzochid_client_socket_unref (sock);
     }
 
   return TRUE;
@@ -356,7 +357,7 @@ void gzochid_server_socket_listen
   server_source = g_io_create_watch (sock->channel, G_IO_IN);
   
   assert (sock->server == NULL);
-  sock->server = g_object_ref (server);
+  sock->server = server;
   
   addr.sin_family = AF_INET;
   addr.sin_port = htons (port);
@@ -394,6 +395,7 @@ void gzochid_server_socket_listen
   g_source_set_callback
     (server_source, (GSourceFunc) dispatch_accept, sock, free_server_socket);
   g_source_attach (server_source, server->main_context);
+  g_source_unref (server_source);
 }
 
 static gpointer 
@@ -452,11 +454,14 @@ _gzochid_client_socket_get_protocol_data (gzochid_client_socket *sock)
 }
 
 void
-gzochid_client_socket_write (gzochid_client_socket *sock, unsigned char *data,
-			     size_t len)
+gzochid_client_socket_write (gzochid_client_socket *sock,
+			     const unsigned char *data, size_t len)
 {
+  assert (sock->server != NULL);
+  
   g_mutex_lock (&sock->sock_mutex);
   g_byte_array_append (sock->send_buffer, data, len);
+
   if (sock->write_source == NULL)
     {
       sock->write_source = g_io_create_watch (sock->channel, G_IO_OUT);
@@ -464,18 +469,224 @@ gzochid_client_socket_write (gzochid_client_socket *sock, unsigned char *data,
 	(sock->write_source, (GSourceFunc) dispatch_client_write, sock, NULL);
       g_source_attach (sock->write_source, sock->server->main_context);
     }
+
   g_mutex_unlock (&sock->sock_mutex);
 }
 
-void
-gzochid_client_socket_free (gzochid_client_socket *sock)
+gzochid_client_socket *
+gzochid_client_socket_ref (gzochid_client_socket *sock)
 {
-  /* It's not safe to free the socket synchronously, because another thread may
-     be in the process of dispatching a read or a write and the caller has no
-     way of knowing that. Instead, unref both sources and let the 
-     `GDestroyNotify' attached to the read source take care of the cleanup. */
-
-  if (sock->write_source != NULL)
-    g_source_unref (sock->write_source);
-  g_source_unref (sock->read_source);
+  g_atomic_int_inc (&sock->ref_count);
+  return sock;
+}
+
+void
+gzochid_client_socket_unref (gzochid_client_socket *sock)
+{
+  if (g_atomic_int_dec_and_test (&sock->ref_count))
+    {
+      /* It's not safe to free the socket synchronously if it's been added to a
+	 main loop, because another thread may be in the process of dispatching
+	 a read or a write and the caller has no way of knowing that. Instead, 
+	 unref both sources and let the `GDestroyNotify' attached to the read 
+	 source take care of the cleanup. */
+
+      /* Need hold the socket mutex in case we're in the middle of a write. */
+      
+      g_mutex_lock (&sock->sock_mutex);
+      
+      if (sock->write_source != NULL)
+	{
+	  g_source_destroy (sock->write_source);
+	  g_source_unref (sock->write_source);
+	}
+      
+      g_mutex_unlock (&sock->sock_mutex);
+      
+      if (sock->read_source != NULL)
+	{
+	  if (! g_source_is_destroyed (sock->read_source))
+	    g_source_destroy (sock->read_source);
+	  g_source_unref (sock->read_source);
+	}
+      
+      sock->protocol.free (sock->protocol_data);
+      
+      g_io_channel_unref (sock->channel);
+      g_mutex_clear (&sock->sock_mutex);
+
+      g_byte_array_unref (sock->recv_buffer);
+      g_byte_array_unref (sock->send_buffer);
+
+      g_free (sock->connection_description);
+
+      free (sock);
+    }
+}
+
+/* A pair of lifecycle callbacks for a `gzochid_reconnectable_socket'. */
+
+struct _gzochid_reconnectable_socket_listener
+{
+  /* The "connected" callback. */
+
+  gzochid_reconnectable_socket_connected connected; 
+
+  gpointer connected_user_data; /* Closure data for the "conected" callback. */
+
+  /* The "connected" callback. */
+
+  gzochid_reconnectable_socket_disconnected disconnected;
+
+  /* Closure data for the "disconnected" callback. */
+
+  gpointer disconnected_user_data; 
+};
+
+typedef struct _gzochid_reconnectable_socket_listener
+gzochid_reconnectable_socket_listener;
+
+/* The reconnectable socket struct. */
+
+struct _gzochid_reconnectable_socket
+{
+  gzochid_client_socket *socket; /* The underlying connection, or `NULL'. */
+  GByteArray *outbound_messages; /* The outbound message buffer. */
+  GList *listeners; /* List of `gzochid_reconnectable_socket_listener'. */
+  
+  /* Protects the relationship between the `socket' and `outbound_messages'
+     fields. This is a recursive mutex to allow lifecycle listeners to call
+     `gzochid_reconnectable_socket_write without blocking. */
+
+  GRecMutex mutex;
+};
+
+gzochid_reconnectable_socket *
+gzochid_reconnectable_socket_new ()
+{
+  gzochid_reconnectable_socket *sock =
+    malloc (sizeof (gzochid_reconnectable_socket));
+
+  sock->socket = NULL;
+  sock->outbound_messages = g_byte_array_new ();
+  sock->listeners = NULL;
+  
+  g_rec_mutex_init (&sock->mutex);
+  
+  return sock;
+}
+
+void
+gzochid_reconnectable_socket_free (gzochid_reconnectable_socket *sock)
+{
+  if (sock->socket != NULL)
+    gzochid_client_socket_unref (sock->socket);
+    
+  g_byte_array_unref (sock->outbound_messages);
+  g_list_free_full (sock->listeners, free);
+  
+  g_rec_mutex_clear (&sock->mutex);
+  
+  free (sock);
+}
+
+void
+gzochid_reconnectable_socket_listen
+(gzochid_reconnectable_socket *sock,
+ gzochid_reconnectable_socket_connected connected, gpointer connected_user_data,
+ gzochid_reconnectable_socket_disconnected disconnected,
+ gpointer disconnected_user_data)
+{
+  gzochid_reconnectable_socket_listener *listener =
+    malloc (sizeof (gzochid_reconnectable_socket_listener));
+
+  listener->connected = connected;
+  listener->connected_user_data = connected_user_data;
+  listener->disconnected = disconnected;
+  listener->disconnected_user_data = disconnected_user_data;
+  
+  g_rec_mutex_lock (&sock->mutex);
+
+  sock->listeners = g_list_append (sock->listeners, listener);
+  
+  g_rec_mutex_unlock (&sock->mutex);
+}
+
+void
+gzochid_reconnectable_socket_write (gzochid_reconnectable_socket *sock,
+				    unsigned char *buf, size_t len)
+{
+  g_rec_mutex_lock (&sock->mutex);
+
+  if (sock->socket != NULL)
+
+    /* If the socket's connected, delegate to the regular socket API. */
+    
+    gzochid_client_socket_write (sock->socket, buf, len);
+
+  /* Otherise, enqueue it in the outbound buffer. */
+  
+  else g_byte_array_append (sock->outbound_messages, buf, len);
+  
+  g_rec_mutex_unlock (&sock->mutex);
+}
+
+void
+gzochid_reconnectable_socket_connect (gzochid_reconnectable_socket *sock,
+				      gzochid_client_socket *client_socket)
+{
+  GList *listener_ptr = NULL;
+  
+  g_rec_mutex_lock (&sock->mutex);
+
+  assert (sock->socket == NULL);
+  sock->socket = gzochid_client_socket_ref (client_socket);
+
+  listener_ptr = sock->listeners;
+  
+  while (listener_ptr != NULL)
+    {
+      gzochid_reconnectable_socket_listener *listener = listener_ptr->data;
+
+      listener->connected (listener->connected_user_data);      
+      listener_ptr = listener_ptr->next;
+    }
+
+  if (sock->outbound_messages->len > 0)
+    {
+      /* Flush the outbound message buffer. */
+      
+      gzochid_client_socket_write
+	(sock->socket, sock->outbound_messages->data,
+	 sock->outbound_messages->len);
+
+      g_byte_array_remove_range
+	(sock->outbound_messages, 0, sock->outbound_messages->len);
+    }
+  
+  g_rec_mutex_unlock (&sock->mutex);
+}
+
+void
+gzochid_reconnectable_socket_disconnect (gzochid_reconnectable_socket *sock)
+{
+  GList *listener_ptr = NULL;
+
+  g_rec_mutex_lock (&sock->mutex);
+
+  assert (sock->socket != NULL);
+  gzochid_client_socket_unref (sock->socket);
+  sock->socket = NULL;
+
+  listener_ptr = sock->listeners;
+  
+  while (listener_ptr != NULL)
+    {
+      gzochid_reconnectable_socket_listener *listener = listener_ptr->data;
+
+      listener->disconnected (listener->disconnected_user_data);      
+      listener_ptr = listener_ptr->next;
+    }
+
+  g_rec_mutex_unlock (&sock->mutex);
 }
