@@ -1,5 +1,5 @@
 /* stats.c: Application statistics management routines for gzochid
- * Copyright (C) 2015 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -16,6 +16,8 @@
  */
 
 #include <assert.h>
+#include <glib.h>
+#include <glib-object.h>
 #include <sys/time.h>
 
 #include "event.h"
@@ -23,21 +25,23 @@
 
 static void
 update_from_data_event (gzochid_application_stats *stats,
-			gzochid_application_event_type type,
-			gzochid_application_data_event *event)
+			gzochid_data_event_type type, GzochidDataEvent *event)
 {
+  guint64 bytes = 0;
+
+  g_object_get (event, "bytes", &bytes, NULL);
+  
   switch (type)
     {
-    case BYTES_READ: stats->bytes_read += event->bytes; break;
-    case BYTES_WRITTEN: stats->bytes_written += event->bytes; break;
+    case BYTES_READ: stats->bytes_read += bytes; break;
+    case BYTES_WRITTEN: stats->bytes_written += bytes; break;
     default: assert (1 == 0);
     }
 }
 
 static void
 update_from_event (gzochid_application_stats *stats,
-		   gzochid_application_event_type type,
-		   gzochid_application_event *event)
+		   gzochid_event_type type, GzochidEvent *event)
 {
   switch (type)
     {
@@ -48,40 +52,35 @@ update_from_event (gzochid_application_stats *stats,
     }
 }
 
-static long
-timeval_to_milliseconds (struct timeval tv)
-{
-  return tv.tv_sec * 1000 + tv.tv_usec / 1000;
-}
-
 static void
 update_from_transaction_event (gzochid_application_stats *stats,
-			       gzochid_application_event_type type,
-			       gzochid_application_transaction_event *event)
+			       gzochid_transaction_event_type type,
+			       GzochidTransactionEvent *event)
 {
-  long transaction_ms = 0;
+  guint64 duration_us = 0;
+  guint64 duration_ms = 0;
+  
+  g_object_get (event, "duration-us", &duration_us, NULL);
 
+  duration_ms = duration_us / 1000;
+  
   switch (type)
     {
-    case TRANSACTION_START: stats->num_transactions_started++; break;
     case TRANSACTION_COMMIT: 
       stats->num_transactions_committed++;
       
-      transaction_ms = timeval_to_milliseconds (event->duration);
-
       if (stats->num_transactions_committed == 1
-	  || transaction_ms > stats->max_transaction_duration)
-	stats->max_transaction_duration = transaction_ms;
+	  || duration_ms > stats->max_transaction_duration)
+	stats->max_transaction_duration = duration_ms;
       if (stats->num_transactions_committed == 1
-	  || transaction_ms < stats->min_transaction_duration)
-	stats->min_transaction_duration = transaction_ms;
+	  || duration_ms < stats->min_transaction_duration)
+	stats->min_transaction_duration = duration_ms;
 
       if (stats->num_transactions_committed == 1)
-	stats->average_transaction_duration = transaction_ms;
+	stats->average_transaction_duration = duration_ms;
       else stats->average_transaction_duration = 
-	     (transaction_ms + 
-	      ((stats->num_transactions_committed - 1)
-	       * stats->average_transaction_duration))
+	     (duration_ms + ((stats->num_transactions_committed - 1)
+			     * stats->average_transaction_duration))
 	     / stats->num_transactions_committed;
       
       break;
@@ -92,26 +91,18 @@ update_from_transaction_event (gzochid_application_stats *stats,
 
 void
 gzochid_stats_update_from_event (gzochid_application_stats *stats,
-				 gzochid_application_event *event)
+				 GzochidEvent *event)
 {
-  switch (event->type)
-    {
-    case BYTES_READ:
-    case BYTES_WRITTEN:
-      update_from_data_event
-	(stats, event->type, (gzochid_application_data_event *) event);
-      break;
+  int type = 0;
+  GType event_type = G_OBJECT_TYPE (event);
 
-    case MESSAGE_RECEIVED:
-    case MESSAGE_SENT:
-    case TRANSACTION_START:
-      update_from_event	(stats, event->type, event);
-      break;
-      
-    case TRANSACTION_COMMIT:
-    case TRANSACTION_ROLLBACK:
-      update_from_transaction_event
-	(stats, event->type, (gzochid_application_transaction_event *) event);
-      break;
-    }
+  g_object_get (event, "type", &type, NULL);
+  
+  if (event_type == GZOCHID_TYPE_DATA_EVENT)
+    update_from_data_event (stats, type, GZOCHID_DATA_EVENT (event));
+  else if (event_type == GZOCHID_TYPE_EVENT)
+    update_from_event (stats, type, event);
+  else if (event_type == GZOCHID_TYPE_TRANSACTION_EVENT)
+    update_from_transaction_event
+      (stats, type, GZOCHID_TRANSACTION_EVENT (event));
 }
