@@ -1,5 +1,5 @@
 /* txlog.c: Transactional log routines for gzochid
- * Copyright (C) 2011 Julian Graham
+ * Copyright (C) 2016 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -18,28 +18,32 @@
 #include <glib.h>
 #include <gzochi-common.h>
 #include <stdarg.h>
+#include <stddef.h>
 #include <stdlib.h>
 #include <syslog.h>
 
 #include "app.h"
-#include "log.h"
 #include "tx.h"
 #include "txlog.h"
 
-typedef struct _gzochid_log_message 
+struct _gzochid_log_message 
 {
-  int priority;
+  GLogLevelFlags priority;
   char *msg;
-} gzochid_log_message;
+};
 
-typedef struct _gzochid_log_transaction_context
+typedef struct _gzochid_log_message gzochid_log_message;
+
+struct _gzochid_log_transaction_context
 {
   gzochid_application_context *context;
   GList *messages;
-} gzochid_log_transaction_context;
+};
 
-static gzochid_log_transaction_context *create_transaction_context
-(gzochid_application_context *context)
+typedef struct _gzochid_log_transaction_context gzochid_log_transaction_context;
+
+static gzochid_log_transaction_context *
+create_transaction_context (gzochid_application_context *context)
 {
   gzochid_log_transaction_context *tx_context =
     calloc (1, sizeof (gzochid_log_transaction_context));
@@ -47,7 +51,8 @@ static gzochid_log_transaction_context *create_transaction_context
   return tx_context;
 }
 
-static gzochid_log_message *gzochid_log_message_new (int priority, char *msg)
+static gzochid_log_message *
+gzochid_log_message_new (int priority, char *msg)
 {
   gzochid_log_message *message = malloc (sizeof (gzochid_log_message));
 
@@ -57,34 +62,38 @@ static gzochid_log_message *gzochid_log_message_new (int priority, char *msg)
   return message;
 }
 
-static void gzochid_log_message_free (gzochid_log_message *message)
+static void
+gzochid_log_message_free (gzochid_log_message *message)
 {
   g_free (message->msg);
   free (message);
 }
 
-static int log_prepare (gpointer data)
+static int
+log_prepare (gpointer data)
 {
   return TRUE;
 }
 
-static void cleanup_transaction (gzochid_log_transaction_context *tx_context)
+static void
+cleanup_transaction (gzochid_log_transaction_context *tx_context)
 {
   g_list_free_full
     (tx_context->messages, (void (*) (gpointer)) gzochid_log_message_free);
   free (tx_context);
 }
 
-static void commit_message (gpointer data, gpointer user_data)
+static void
+commit_message (gpointer data, gpointer user_data)
 {
-  gzochid_log_message *message = (gzochid_log_message *) data;
-  gzochid_log (message->priority, message->msg);
+  gzochid_log_message *message = data;
+  g_log (G_LOG_DOMAIN, message->priority, "%s", message->msg);
 }
 
-static void log_commit (gpointer data)
+static void
+log_commit (gpointer data)
 {
-  gzochid_log_transaction_context *tx_context =
-    (gzochid_log_transaction_context *) data;
+  gzochid_log_transaction_context *tx_context = data;
   
   g_list_foreach 
     (tx_context->messages, (void (*) (gpointer, gpointer)) commit_message, 
@@ -92,10 +101,10 @@ static void log_commit (gpointer data)
   cleanup_transaction (tx_context);
 }
 
-static void log_rollback (gpointer data)
+static void
+log_rollback (gpointer data)
 {
-  gzochid_log_transaction_context *tx_context =
-    (gzochid_log_transaction_context *) data;
+  gzochid_log_transaction_context *tx_context = data;
 
   cleanup_transaction (tx_context);
 }
@@ -103,19 +112,19 @@ static void log_rollback (gpointer data)
 static gzochid_transaction_participant log_participant =
   { "log", log_prepare, log_commit, log_rollback };
 
-static gzochid_log_transaction_context *join_transaction 
-(gzochid_application_context *context)
+static gzochid_log_transaction_context *
+join_transaction (gzochid_application_context *context)
 {
   if (!gzochid_transaction_active()
       || gzochid_transaction_context (&log_participant) == NULL)
     gzochid_transaction_join
       (&log_participant, create_transaction_context (context));
-  return (gzochid_log_transaction_context *) 
-    gzochid_transaction_context (&log_participant);
+  return gzochid_transaction_context (&log_participant);
 }
 
-static void gzochid_tx_vlog 
-(gzochid_application_context *context, int priority, char *msg, va_list ap)
+static void
+gzochid_tx_vlog (gzochid_application_context *context, int priority, char *msg,
+		 va_list ap)
 {
   va_list app;
   va_copy (app, ap);
@@ -128,8 +137,9 @@ static void gzochid_tx_vlog
   va_end (app);
 }
 
-void gzochid_tx_log
-(gzochid_application_context *context, int priority, char *msg, ...)
+void
+gzochid_tx_log (gzochid_application_context *context, GLogLevelFlags priority,
+		char *msg, ...)
 {
   va_list args;
   va_start (args, msg);
@@ -137,42 +147,47 @@ void gzochid_tx_log
   va_end (args);
 }
 
-void gzochid_tx_err (gzochid_application_context *context, char *msg, ...)
+void
+gzochid_tx_err (gzochid_application_context *context, char *msg, ...)
 {
   va_list args;
   va_start (args, msg);
-  gzochid_tx_vlog (context, LOG_ERR, msg, args);
+  gzochid_tx_vlog (context, G_LOG_LEVEL_CRITICAL, msg, args);
   va_end (args);
 }
 
-void gzochid_tx_warning (gzochid_application_context *context, char *msg, ...)
+void
+gzochid_tx_warning (gzochid_application_context *context, char *msg, ...)
 {
   va_list args;
   va_start (args, msg);
-  gzochid_tx_vlog (context, LOG_WARNING, msg, args);
+  gzochid_tx_vlog (context, G_LOG_LEVEL_WARNING, msg, args);
   va_end (args);
 }
 
-void gzochid_tx_notice (gzochid_application_context *context, char *msg, ...)
+void
+gzochid_tx_notice (gzochid_application_context *context, char *msg, ...)
 {
   va_list args;
   va_start (args, msg);
-  gzochid_tx_vlog (context, LOG_NOTICE, msg, args);
+  gzochid_tx_vlog (context, G_LOG_LEVEL_MESSAGE, msg, args);
   va_end (args);
 }
 
-void gzochid_tx_info (gzochid_application_context *context, char *msg, ...)
+void
+gzochid_tx_info (gzochid_application_context *context, char *msg, ...)
 {
   va_list args;
   va_start (args, msg);
-  gzochid_tx_vlog (context, LOG_INFO, msg, args);
+  gzochid_tx_vlog (context, G_LOG_LEVEL_INFO, msg, args);
   va_end (args);
 }
 
-void gzochid_tx_debug (gzochid_application_context *context, char *msg, ...)
+void
+gzochid_tx_debug (gzochid_application_context *context, char *msg, ...)
 {
   va_list args;
   va_start (args, msg);
-  gzochid_tx_vlog (context, LOG_DEBUG, msg, args);
+  gzochid_tx_vlog (context, G_LOG_LEVEL_DEBUG, msg, args);
   va_end (args);
 }
