@@ -1,5 +1,5 @@
 /* api/test-tx.c: Test routines for api/tx.c in gzochid.
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include "../app.h"
 #include "../channel.h"
 #include "../context.h"
+#include "../descriptor.h"
 #include "../guile.h"
 #include "../gzochid-auth.h"
 #include "../session.h"
@@ -31,6 +32,42 @@
 #include "tx.h"
 #include "util.h"
 
+/* Fake implementations to avoid having to pull in `app.o'. */
+
+static gzochid_application_context *current_app_context = NULL;
+
+gzochid_application_context *
+gzochid_application_context_new ()
+{
+  return calloc (1, sizeof (gzochid_application_context));
+}
+
+void
+gzochid_application_context_free (gzochid_application_context *app_context)
+{
+  free (app_context);
+}
+
+void *
+gzochid_with_application_context (gzochid_application_context *app_context,
+				  gzochid_auth_identity *identity,
+				  void *(*worker) (gpointer), gpointer data)
+{
+  gpointer ret = NULL;
+  
+  current_app_context = app_context;
+  ret = worker (data);
+  current_app_context = NULL;
+  
+  return ret;
+}
+
+gzochid_application_context *
+gzochid_get_current_application_context ()
+{
+  return current_app_context;
+}
+
 /* Fake implementation to avoid having to pull in `channel.o'. */
 
 const char *
@@ -348,8 +385,8 @@ inner_main (void *data, int argc, char *argv[])
   
   t.app_context = gzochid_application_context_new ();
   t.app_context->identity_cache = gzochid_auth_identity_cache_new ();
-  t.app_context->descriptor = 
-    calloc (1, sizeof (gzochid_application_descriptor));
+  t.app_context->descriptor =
+    g_object_new (GZOCHID_TYPE_APPLICATION_DESCRIPTOR, NULL);
   t.app_context->deployment_root = "";
 
   application_context_init (t.app_context);
@@ -369,7 +406,7 @@ inner_main (void *data, int argc, char *argv[])
   g_test_run ();
 
   gzochid_auth_identity_unref (t.identity);
-  free (t.app_context->descriptor);
+  g_object_unref (t.app_context->descriptor);
   gzochid_auth_identity_cache_destroy (t.app_context->identity_cache);
   gzochid_application_context_free (t.app_context);
 }
