@@ -32,6 +32,31 @@ struct _GzochidDataClient
 
 G_DEFINE_TYPE (GzochidDataClient, gzochid_data_client, G_TYPE_OBJECT);
 
+enum test_dataclient_properties
+  {
+    PROP_CONNECTION_DESCRIPTION = 1,
+    N_PROPERTIES
+  };
+
+static GParamSpec *obj_properties[N_PROPERTIES] = { NULL };
+
+static void
+gzochid_data_client_get_property (GObject *object, guint property_id,
+				  GValue *value, GParamSpec *pspec)
+{
+  GzochidDataClient *data_client = GZOCHID_DATA_CLIENT (object);
+
+  switch (property_id)
+    {
+    case PROP_CONNECTION_DESCRIPTION:
+      g_value_set_static_string (value, "127.0.0.1");
+      break;
+      
+    default:
+      g_test_fail ();
+    }
+}
+
 static void
 gzochid_data_client_finalize (GObject *gobject)
 {
@@ -46,12 +71,19 @@ gzochid_data_client_class_init (GzochidDataClientClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   object_class->finalize = gzochid_data_client_finalize;
+  object_class->get_property = gzochid_data_client_get_property;
+
+  obj_properties[PROP_CONNECTION_DESCRIPTION] = g_param_spec_string
+    ("connection-description", "conn-desc", "Test connection description", NULL,
+     G_PARAM_READABLE);
+
+  g_object_class_install_properties
+    (object_class, N_PROPERTIES, obj_properties);
 }
 
 static void
 gzochid_data_client_init (GzochidDataClient *self)
 {
-  self->connected = TRUE;
 }
 
 void
@@ -203,17 +235,6 @@ test_client_dispatch_multiple ()
   g_object_unref (client);
 }
 
-static void
-test_client_error ()
-{
-  GzochidDataClient *client = g_object_new (GZOCHID_TYPE_DATA_CLIENT, NULL);
-  
-  gzochid_dataclient_client_protocol.error (client);
-  g_assert (! client->connected);
-  
-  g_object_unref (client);
-}
-
 int
 main (int argc, char *argv[])
 {
@@ -240,7 +261,6 @@ main (int argc, char *argv[])
      test_client_dispatch_one_next_key_response);
 
   g_test_add_func ("/client/dispatch/multiple", test_client_dispatch_multiple);
-  g_test_add_func ("/client/error", test_client_error);
   
   return g_test_run ();
 }
