@@ -1,5 +1,5 @@
 /* test-dataserver.c: Tests for dataserver.c in gzochi-metad.
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -137,11 +137,12 @@ setup_dataserver (dataserver_fixture *fixture, gconstpointer user_data)
   
   g_assert_no_error (err);
   
-  g_key_file_unref (key_file);
   g_object_unref (resolution_context);
   g_object_unref (configuration);
   
   gzochi_metad_dataserver_start (fixture->server);
+  
+  g_key_file_unref (key_file);
 }
 
 static void
@@ -426,6 +427,18 @@ test_release_all (dataserver_fixture *fixture, gconstpointer user_data)
   g_bytes_unref (key2);
 }
 
+static void
+obtain_lock (GzochiMetadDataServer *server, const char *app, const char *store,
+	     GBytes *key)
+{
+  GError *err = NULL;
+  gzochid_data_response *response =  gzochi_metad_dataserver_request_value
+    (server, 1, app, store, key, TRUE, &err);
+
+  g_assert_no_error (err);
+  gzochid_data_response_free (response);
+}
+
 static void
 test_process_changeset (dataserver_fixture *fixture, gconstpointer user_data)
 {
@@ -491,6 +504,14 @@ test_process_changeset (dataserver_fixture *fixture, gconstpointer user_data)
   test_storage_open (test_context, "oids", GZOCHID_STORAGE_CREATE);
   test_storage_open (test_context, "names", GZOCHID_STORAGE_CREATE);
 
+  obtain_lock (fixture->server, "test", "oids", object_change1.key);
+  obtain_lock (fixture->server, "test", "oids", object_change2.key);
+  obtain_lock (fixture->server, "test", "oids", object_change3.key);
+
+  obtain_lock (fixture->server, "test", "names", binding_change1.key);
+  obtain_lock (fixture->server, "test", "names", binding_change2.key);
+  obtain_lock (fixture->server, "test", "names", binding_change3.key);
+  
   put (oids, "1", 2, "foo", 4);
   put (oids, "2", 2, "bar", 4);
 
@@ -582,7 +603,7 @@ main (int argc, char *argv[])
     test_storage_close_context;
   gzochid_storage_engine_interface_mem.open = test_storage_open;
   gzochid_storage_engine_interface_mem.close_store = test_storage_close_store;
-  
+
   g_test_add ("/dataserver/reserve-oids", dataserver_fixture, NULL,
 	      setup_dataserver, test_reserve_oids, teardown_dataserver);
   g_test_add ("/dataserver/request-value", dataserver_fixture, NULL,
