@@ -1,5 +1,5 @@
 /* test-scheme.c: Test routines for scheme.c in gzochid.
- * Copyright (C) 2016 Julian Graham
+ * Copyright (C) 2017 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -22,12 +22,43 @@
 
 #include "app.h"
 #include "channel.h"
+#include "descriptor.h"
 #include "guile.h"
 #include "gzochid-auth.h"
 #include "scheme.h"
 #include "session.h"
 #include "util.h"
 
+/* Fake implementations to avoid having to pull in `app.o'. */
+
+static gzochid_application_context *current_app_context = NULL;
+
+gzochid_application_context *
+gzochid_application_context_new ()
+{
+  return calloc (1, sizeof (gzochid_application_context));
+}
+
+void
+gzochid_application_context_free (gzochid_application_context *app_context)
+{
+  free (app_context);
+}
+
+void *
+gzochid_with_application_context (gzochid_application_context *app_context,
+				  gzochid_auth_identity *identity,
+				  void *(*thunk) (gpointer), gpointer data)
+{
+  gpointer ret = NULL;
+  
+  current_app_context = app_context;
+  ret = thunk (data);
+  current_app_context = NULL;
+  
+  return ret;
+}
+
 /* Fake implementation to avoid having to pull in `channel.o'. */
 
 const char *
@@ -51,9 +82,9 @@ test_serialization_exception ()
   GByteArray *str = g_byte_array_new ();
   GError *err = NULL;
 
-  context->descriptor = calloc (1, sizeof (gzochid_application_descriptor));
+  context->descriptor = g_object_new
+    (GZOCHID_TYPE_APPLICATION_DESCRIPTOR, NULL);
   context->deployment_root = "/";
-  g_mutex_init (&((gzochid_context *) context)->mutex);
   
   gzochid_util_serialize_string ("test-unknown-type", str);
 
@@ -64,7 +95,7 @@ test_serialization_exception ()
   g_assert_error (err, GZOCHID_SCHEME_ERROR, GZOCHID_SCHEME_ERROR_FAILED);
   g_clear_error (&err);
 
-  free (context->descriptor);
+  g_object_unref (context->descriptor);
   gzochid_application_context_free (context);
   g_byte_array_unref (str);
 }
@@ -76,7 +107,6 @@ test_serialization_corrupt ()
   GByteArray *str = g_byte_array_new ();
   GError *err = NULL;
 
-  g_mutex_init (&((gzochid_context *) context)->mutex);
   gzochid_util_serialize_int (16, str);  
 
   g_assert
@@ -86,7 +116,6 @@ test_serialization_corrupt ()
   g_assert_error (err, GZOCHID_SCHEME_ERROR, GZOCHID_SCHEME_ERROR_SERIAL);
   g_clear_error (&err);
 
-  free (context->descriptor);
   gzochid_application_context_free (context);
   g_byte_array_unref (str);
 }
