@@ -153,19 +153,25 @@ process_response_async (gpointer data)
       closure->success_callback
 	(closure->response->data, closure->success_data);
 
+      g_mutex_lock (&client->process_response_mutex);  
+
       client->release_closures =
 	g_list_append (client->release_closures, 
 		       create_release_closure (closure->qualified_key,
 					       closure->release_callback,
 					       closure->release_data));      
     }
-  else closure->failure_callback
-	 (closure->response->timeout, closure->failure_data);
+  else
+    {
+      closure->failure_callback
+	(closure->response->timeout, closure->failure_data);
+
+      g_mutex_lock (&client->process_response_mutex);  
+    }
       
   g_free (closure->qualified_key);
   free (closure);
 
-  g_mutex_lock (&client->process_response_mutex);  
   g_cond_signal (&client->process_response_cond);
   g_mutex_unlock (&client->process_response_mutex);
   
@@ -1433,8 +1439,16 @@ test_cache_store_consistency (dataclient_storage_fixture *fixture,
   g_mutex_unlock (&fixture->dataclient->process_response_mutex);
   
   free (value);
+
+  g_mutex_lock (&fixture->dataclient->process_response_mutex);
+  
   fixture->iface->transaction_put (tx, fixture->store, "foo", 4, "baz", 4);
 
+  g_cond_wait (&fixture->dataclient->process_response_cond,
+	       &fixture->dataclient->process_response_mutex);
+  
+  g_mutex_unlock (&fixture->dataclient->process_response_mutex);
+  
   fixture->iface->transaction_prepare (tx);
   fixture->iface->transaction_commit (tx);
 
